#include <stdexcept>
#include "SDL.h"
#include "actioneventqueue.h"
#include "ccmap.h"
#include "common.h"
#include "ai.h"
#include "playerpool.h"
#include "unit.h"
#include "structure.h"
#include "unitandstructurepool.h"
#include "weaponspool.h"
#include "soundengine.h"
#include "dispatcher.h"

#define DEBUG_AI

using p::ccmap;

// AI constructor
ai::ai()
{
    this->Difficulty = DIFFICUTY_EASY;
    LplayerBasePos = 0;
	UnitBuildMultiplexer = 0;
	guide = false;

	NumbPlayers 	= p::ppool->getNumPlayers();

	for (unsigned int i = 0; i < (unsigned)NumbPlayers; i++){
		LastGuideTickCount.push_back (SDL_GetTicks());
		LastBuildTickCount.push_back (SDL_GetTicks());
		LastAttackTickCount.push_back (SDL_GetTicks());
	}
//  AiDifficultyTimer.Start();
}


// AI destructor
ai::~ai()
{


}

Uint32 ai::getDist(Uint32 pos1, Uint32 pos2)
{
    Uint16 x, y, nx, ny, xdiff, ydiff;
    x = pos2%p::ccmap->getWidth();
    y = pos2/p::ccmap->getWidth();
    nx = pos1%p::ccmap->getWidth();
    ny = pos1/p::ccmap->getWidth();

    xdiff = abs(x-nx);
    ydiff = abs(y-ny);
    return min(xdiff,ydiff)+abs(xdiff-ydiff);
}

void ai::SetDifficulty (int Diff)
{
	if (Diff <= DIFFICUTY_VERYHARD)
		this->Difficulty = Diff;
}


//
//	Returns a enemy **unit** if one is in range of our **unit**
//
Unit* ai::EnemyUnitInRange (int MyPlayerNumb, Unit* MyUnit, int AttackRange )
{
Player                  *EnemyPlayer,
                        *MyPlayer;
int                     EnemyNumbUnits;
std::vector<Unit*>      Enemyunitpool;
Unit*                   EnemyUnit;

	if (MyUnit == NULL)
		return NULL;

    if (!MyUnit->canAttack())
        return NULL;
#if 1
    if (AttackRange == -1)
        AttackRange = MyUnit->getType()->getWeapon()->getRange();    //  MyUnit->getType()->getSight();
#else
    if (AttackRange == -1)
        AttackRange = MyUnit->getType()->getSight();

#endif

    for (int i = 0; i < this->NumbPlayers; i++){
        // Don't find my own units
        if (MyPlayerNumb == i)
            continue;

        EnemyPlayer = p::ppool->getPlayer(i);

        MyPlayer = p::ppool->getPlayer(MyPlayerNumb);

        // Check if the Enemy player is a allie ;)
        if (EnemyPlayer->isAllied(MyPlayer))
            continue;

        EnemyNumbUnits = EnemyPlayer->getNumUnits();

        Enemyunitpool = EnemyPlayer->getUnits();

        // For each unit from this player
        for (int UnitNumb = 0; UnitNumb < EnemyNumbUnits; UnitNumb++){
            EnemyUnit = Enemyunitpool[UnitNumb];
            int distance = MyUnit->getDist(EnemyUnit->getPos());
            if (distance <= AttackRange && EnemyPlayer->getSide() != PS_NEUTRAL){
                return EnemyUnit;
            }
        }
    }
    return NULL;
}

//
//	Returns a enemy **structure** if one is in range of our **unit**
//
Structure* ai::EnemyStructureInRange (int MyPlayerNumb, Unit* MyUnit, int AttackRange )
{
Player                  *EnemyPlayer,
                        *MyPlayer;
int                     EnemyNumbStructures;
std::vector<Structure*>	Enemystructurepool;
Structure*              EnemyStructure;

	if (MyUnit == NULL)
		return NULL;

    if (!MyUnit->canAttack())
        return NULL;

    if (AttackRange == -1)
        AttackRange = MyUnit->getType()->getWeapon()->getRange();  //MyUnit->getType()->getSight();

    for (int i = 0; i < this->NumbPlayers; i++){
        // Don't find my own units
        if (MyPlayerNumb == i)
            continue;

        EnemyPlayer = p::ppool->getPlayer(i);

        MyPlayer = p::ppool->getPlayer(MyPlayerNumb);

        // Check if the Enemy player is a allie ;)
        if (EnemyPlayer->isAllied(MyPlayer))
            continue;

        EnemyNumbStructures	= EnemyPlayer->getNumStructs();	
        Enemystructurepool	= EnemyPlayer->getStructures();

        // For each unit from this player
        for (int StructureNumb = 0; StructureNumb < EnemyNumbStructures; StructureNumb++){
            EnemyStructure = Enemystructurepool[StructureNumb];
            int distance = MyUnit->getDist(EnemyStructure->getPos());
            if (distance <= AttackRange && EnemyPlayer->getSide() != PS_NEUTRAL){
                return EnemyStructure;
            }
        }
    }
    return NULL;
}

//
//	Returns a enemy **unit** if one is in range of our **structure**
//
Unit* ai::EnemyUnitInRange (int MyPlayerNumb, Structure* MyStructure, int AttackRange )
{
Player              *EnemyPlayer, *MyPlayer;
int                 EnemyNumbUnits;
std::vector<Unit*>	Enemyunitpool;
Unit*               EnemyUnit;

	if (MyStructure == NULL)
		return NULL;

	if (!MyStructure->canAttack())
		return NULL;

	if (AttackRange == -1)
		AttackRange = MyStructure->getType()->getWeapon(true)->getRange();

	for (int i = 0; i < this->NumbPlayers; i++){

		// Don't defend against my own units
		if (MyPlayerNumb == i)
			continue;

		EnemyPlayer = p::ppool->getPlayer(i);

		MyPlayer = p::ppool->getPlayer(MyPlayerNumb);

		// Check if the Enemy player is a allie ;)
		if (EnemyPlayer->isAllied(MyPlayer))
			continue;

		EnemyNumbUnits = EnemyPlayer->getNumUnits();
			
		Enemyunitpool = EnemyPlayer->getUnits();

		// For each unit from this player
		for (int UnitNumb = 0; UnitNumb < EnemyNumbUnits; UnitNumb++){
			EnemyUnit = Enemyunitpool[UnitNumb];
//			int distance = MyStructure->getDist(EnemyUnit->getPos());
			int distance = EnemyUnit->getDist(MyStructure->getPos());
			if (distance <= AttackRange  && EnemyPlayer->getSide() != PS_NEUTRAL){
				return EnemyUnit;
			}

		}
	}
	return NULL;
}


unsigned int ai::FindClosesedTiberium (Unit *Unit)
{
Uint32 tiberium;
Uint32 ClosesedPos, ClosesedDistance, Distance;
bool	FirstFound = false;

	for (unsigned int pos =0; pos < p::ccmap->getSize(); pos++){
		tiberium = p::ccmap->getResourceFrame(pos);
		if (tiberium != 0) {
			// Found tiberium
			Distance = Unit->getDist(pos);
			if (Distance < ClosesedDistance || !FirstFound){
				ClosesedPos		= pos;
				ClosesedDistance	= Distance;
				FirstFound = true;
			}
		}
	}
	return ClosesedPos;

}


//
//	Actually this function attacks enemy units and structures in range :)
//
void ai::DefendUnits (Player *Player, int PlayerNumb)
{
UnitOrStructure     *Enemy = NULL;
int                 NumbUnits;
std::vector<Unit*>	unitpool;
Unit                *Unit,
                    *EnemyUnit;
Structure           *EnemyStructure;


	NumbUnits = Player->getNumUnits();
			
	unitpool = Player->getUnits();

	// For each unit from this player
	for (int UnitNumb = 0; UnitNumb < NumbUnits; UnitNumb++){

		Unit = unitpool[UnitNumb];

        // Check that the unit can attack and that the unit doens't have other thing to do
		if (!Unit->canAttack() || Unit->IsAttacking() || Unit->IsMoving ())
			continue;

//		if (PlayerNumb == this->HumanPlayerNumb ){
//			if ( !Unit->underattack())
//				continue;
//		}

		EnemyUnit = EnemyUnitInRange (PlayerNumb, Unit );
		if (EnemyUnit != NULL && Unit->canAttack()){

			Enemy  = p::uspool->getUnitOrStructureAt(EnemyUnit->getPos(), EnemyUnit->getSubpos());

			if (Enemy != NULL){
				Unit->attack(Enemy);
				return;
			}
		}

		EnemyStructure = EnemyStructureInRange ( PlayerNumb, Unit );
		if (EnemyStructure != NULL && Unit->canAttack()){

			Enemy  = p::uspool->getUnitOrStructureAt(EnemyStructure->getPos(), EnemyStructure->getSubpos());

			if ( Enemy != NULL ){
				Unit->attack(Enemy);
				return;
			}
		}
	}
}

void ai::DefendStructures (Player *Player, int PlayerNumb)
{
std::vector<Structure*> structurepool;
UnitOrStructure         *Enemy = NULL;
int                     NumbStructures;
Structure               *Structure;
Unit                    *EnemyUnit;

	//
	// Make structures defend themselfes
	//
	structurepool = Player->getStructures();
	NumbStructures = Player->getNumStructs();

	// For each structure from this player
	for (int StructNumb = 0; StructNumb < NumbStructures; StructNumb++){
		Structure = structurepool[StructNumb];

//        if (Structure->IsBuilding ())
//            continue;

		if (!Structure->canAttack())
			continue;

		// Check if we have enoug power
        StructureType* st = (StructureType*)Structure->getType();
		if ( st->getPowerInfo().powered){
  			if (Player->getPower() < Player->getPowerUsed()){
				continue;
			}
		}

		EnemyUnit = EnemyUnitInRange (PlayerNumb, Structure );

		if (EnemyUnit != NULL && Structure->canAttack()){

			Enemy  = p::uspool->getUnitOrStructureAt(EnemyUnit->getPos(), EnemyUnit->getSubpos());

			if (Enemy != NULL && !Structure->IsAttacking ()){
				Structure->attack(Enemy);
			}
		}
	}
}
#if 0
bool ai::CanBuildAt (Uint8 PlayerNumb, const char *structname, Uint32 pos)
{
Uint32 placeypos, placexpos;
int x, y;
std::vector<bool>& buildable = p::ppool->getPlayer(PlayerNumb)->getMapBuildable();
Uint32 br = pos + ccmap->getWidth()*(type->getYsize()-1);

	StructureType* type = p::uspool->getStructureTypeByName("PROC");

    for (placeypos = 0; placeypos < placetype->getYsize(); placeypos++) {
        for (placexpos = 0; placexpos < placetype->getXsize(); placexpos++) {

	// Check that we don't try to build outside the map (copied check from unitandstructurepool.cpp!!

	if (pos > ccmap->getSize() || (br > ccmap->getSize() && 0))
		return false;

	if (!buildable[pos])
		return false;

	Uint32 curpos = pos+ccmap->getWidth()*(type->getYsize());
	for (y = type->getYsize()-1; y>=0; --y) {
		curpos -= ccmap->getWidth();
		for (x = type->getXsize()-1; x>=0; --x) {
			if (p::uspool->getStructureAt(curpos+x) != NULL || p::uspool->getUnitAt(curpos+x,0) != NULL){    //p::uspool->getUnitAt(curpos+x) != NULL
				return false;
			}
		}
	}

	if (p::ccmap->isBuildableAt(PlayerNumb, pos))
		return true;

	return false;
}
#else
bool ai::CanBuildAt (Uint8 PlayerNumb, const char *structname, Uint32 pos)
{
Uint32	placeypos,
		placexpos,
		curpos;
Uint16	xpos,
		ypos;
Uint32	br;

    p::ccmap->translateFromPos(pos, &xpos, &ypos);

	StructureType* Type		= p::uspool->getStructureTypeByName(structname);

	std::vector<bool>& buildable = p::ppool->getPlayer(PlayerNumb)->getMapBuildable();

	// Check that we don't try to build outside the map (copied check from unitandstructurepool.cpp!!
	br = pos + ccmap->getWidth()*(Type->getYsize()-1);
	if (pos > ccmap->getSize() || (br > ccmap->getSize() && 0))
		return false;

	// Prevent wrapping around
	if (xpos + Type->getXsize() > p::ccmap->getWidth())
		return false;

	for (placeypos = 0; placeypos < Type->getYsize(); placeypos++) {
		for (placexpos = 0; placexpos < Type->getXsize(); placexpos++) {

            curpos = pos+placeypos*p::ccmap->getWidth()+placexpos;

			if (!buildable[curpos])
				return false;

			if (p::uspool->getStructureAt(curpos) != NULL || p::uspool->getUnitAt(curpos,0) != NULL)    //p::uspool->getUnitAt(curpos+x) != NULL
				return false;

			if (!p::ccmap->isBuildableAt(PlayerNumb, curpos))
				return false;

		}
	}
	return true;
}
#endif

bool ai::BuildStructure (Player *Player, int PlayerNumb, const char *structname, Uint32 ConstYardPos)
{
Uint32 curpos = 0;

	// Get the structure type so we can see what the cost are for this structure
	StructureType* type = p::uspool->getStructureTypeByName(structname);

	//Check if we can pay for this structure ;)
	if (Player->getMoney() > type->getCost()){
		for (Uint32 offset = 5; offset < 100; offset += 4){
//		Uint32 offset = 20;
			curpos = ConstYardPos - ccmap->getWidth()*(offset/2);
			for (Uint32 pos = curpos; pos < curpos + offset; pos +=4){
				if (CanBuildAt (PlayerNumb, structname, pos) ){
					//if (!p::uspool->createStructure(structname, pos, PlayerNumb, FULLHEALTH /*255*/, 0, true))
					if (!p::dispatcher->structurePlace(structname, pos, PlayerNumb))
                        logger->error("Failed to build, ConstYard-pos = %u, pos = %u\n", ConstYardPos, pos);
					else{
						// Succes at building
						return true;
					}
				}
			}
		}
	}
	printf ("%s line %i: Failed to find free pos\n", __FILE__, __LINE__);
	return false;
}

// Send all units to defend if our base is under attack
void ai::DefendComputerPlayerBaseUnderAttack (Player *Player, int PlayerNumb, UnitOrStructure *Enemy, Structure *StructureUnderAttack)
{
int					NumbUnits = 0;
std::vector<Unit*>	unitpool;
Unit				*Unit = NULL;
int					Distance, SightRange, StructureUnderAttackPos;

	if (Enemy == NULL)
		return;

	//
	// don't try to build for the human player
	// We should actually check if building is allowed and only build when allowed (by triggers)
	//
	if (PlayerNumb == this->HumanPlayerNumb){
		return;
	}

	NumbUnits				= Player->getNumUnits();
	unitpool				= Player->getUnits();
	StructureUnderAttackPos	= StructureUnderAttack->getPos();

	for (int UnitNumb = 0; UnitNumb < NumbUnits; UnitNumb++){
		Unit = unitpool[UnitNumb];

		if (/*Unit->IsAttacking () ||*/ !Unit->canAttack())
			continue;

		Distance	= Unit->getDist(StructureUnderAttackPos);
		SightRange	= Unit->getType()->getSight();

		if ( Distance < (SightRange * 3) /*&& !Unit->IsAttacking ()*/){
			Unit->attack ((UnitOrStructure*)Enemy);
		}
	}
}


// Help a computer player unit under attack
void ai::DefendComputerPlayerUnitUnderAttack (Player *Player, int PlayerNumb, UnitOrStructure *Enemy, Unit *UnitUnderAttack)
{
int					NumbUnits = 0;
std::vector<Unit*>	unitpool;
Unit				*Unit = NULL;
int					Distance, SightRange, UnitUnderAttackPos;

	if ( Enemy == NULL )
		return;

	// don't try to build for the human player
	if (PlayerNumb == this->HumanPlayerNumb){
		return;
	}

	// Start with shooting back
	if (UnitUnderAttack->canAttack())
		UnitUnderAttack->attack (Enemy);


	NumbUnits	= Player->getNumUnits();
	unitpool	= Player->getUnits();

	UnitUnderAttackPos = UnitUnderAttack->getPos();

	for (int UnitNumb = 0; UnitNumb < NumbUnits; UnitNumb++){
		Unit = unitpool[UnitNumb];

		if (/*Unit->IsAttacking () ||*/ !Unit->canAttack())
			continue;

		Distance	= Unit->getDist(UnitUnderAttackPos);
		SightRange	= Unit->getType()->getSight();

		// If I can see my comrade under attack
		if ( Distance < SightRange){
			Unit->attack ((UnitOrStructure*)Enemy);
		}
	}

}

//
//	This is a beginning, for now this function only deploy's the starting MCV
//
void ai::Build (Player *Player, int PlayerNumb)
{
std::vector<Structure*>	structurepool;
int					NumbUnits,
					NumbStructures;
std::vector<Unit*>	unitpool;
Unit				*Unit;
Structure			*OreRefinery;
Uint32				NumbWarfactorys = 0, NumbOreRefs = 0, NumbConstYards = 0, NumbBarracks = 0, NumbOfOreTrucks = 0, NumbTanks = 0, NumbTeslaCoils = 0 , NumbOfPowerPlants = 0;
Structure			*Structure;
Uint32				ConstYardPos = 0;
//UnitType			*BuildUnitType;
int					cost;
bool				StructureWasBuild = false;
//static bool		Direction = false;
//static Uint32		MoveToPos = 0;
//char			UnitName[10];

	// Don't build in single player mode (jet).
	if (ccmap->getGameMode() == GAME_MODE_SINGLE_PLAYER){
		return;
	}



	NumbUnits		= Player->getNumUnits();
	structurepool	= Player->getStructures();
	NumbStructures	= Player->getNumStructs();

	unitpool = Player->getUnits();

	NumbWarfactorys = NumbOreRefs = NumbConstYards = NumbBarracks = 0;


	OreRefinery = NULL;
	for (int StructNumb = 0; StructNumb < NumbStructures; StructNumb++){
		Structure = structurepool[StructNumb];

		if (strcmp (Structure->getType()->getTName(), "WEAP") == 0){
			NumbWarfactorys++;
		}

		if (strcmp (Structure->getType()->getTName(), "PROC") == 0){
			NumbOreRefs++;
			OreRefinery = Structure;
		}

		if (strcmp (Structure->getType()->getTName(), "FACT") == 0){
			NumbConstYards++;
			ConstYardPos = Structure->getPos();
		}

		if (strcmp (Structure->getType()->getTName(), "TSLA") == 0){
			NumbTeslaCoils++;
		}

		if (strcmp (Structure->getType()->getTName(), "APWR") == 0){
			NumbOfPowerPlants++;
		}

		if (strcmp (Structure->getType()->getTName(), "TENT") == 0 || strcmp (Structure->getType()->getTName(), "BARR") == 0 ){
			NumbBarracks++;
		}
	}

	for (int UnitNumb = 0; UnitNumb < NumbUnits; UnitNumb++){
		Unit = unitpool[UnitNumb];

		if (strcmp (Unit->getType()->getTName(), "HARV") == 0){
			if (Unit->GetBaseRefinery() == NULL)
				Unit->SetBaseRefinery (OreRefinery);
			NumbOfOreTrucks++;
		}

		if (strcmp (Unit->getType()->getTName(), "1TNK") == 0){
			NumbTanks++;
		}
	}

	// don't try to build for the human player
	if (PlayerNumb == this->HumanPlayerNumb){
		/// TODO find correct soud to play
//		if (BaseUnderAttack != OldBaseUnderAttack[PlayerNumb])
//			pc::sfxeng->PlaySound("baseatk1.aud");
//		OldBaseUnderAttack[PlayerNumb] = BaseUnderAttack; 
		return;
	}

	// Check our units if we can deploy them (only for contruction yard jet)
	if (NumbConstYards == 0){
		for (int UnitNumb = 0; UnitNumb < NumbUnits; UnitNumb++){
			Unit = unitpool[UnitNumb];

			if (Unit->canDeploy()){
				// We don't have any structures, randomly deploy
				Unit->deploy();
			}
		}
	}

	if (NumbConstYards > 0){

		// Build ore refinery
		if (!StructureWasBuild && ((NumbOreRefs == 0 || (NumbOreRefs < 2 && NumbTeslaCoils == 2)) && NumbConstYards > 0)){
			cost = p::uspool->getStructureTypeByName("PROC")->getCost();
			if (Player->getMoney() > cost){
				if (BuildStructure (Player, PlayerNumb, "PROC", ConstYardPos))
					Player->changeMoney(-1 * cost);
				StructureWasBuild = true;
			}
			/// TODO sell some stuff if we can't afford a new refinery and we dont't have any refinery's anymore
		}

		// Build the weapen factory
		if (!StructureWasBuild && (NumbWarfactorys == 0 && NumbConstYards > 0)){
			cost = p::uspool->getStructureTypeByName("WEAP")->getCost();
			if (Player->getMoney() > cost){
				if (BuildStructure (Player, PlayerNumb, "WEAP", ConstYardPos))
					Player->changeMoney(-1 * cost);
				StructureWasBuild = true;
			}
		}

		// build power plant (if needed)
		if (!StructureWasBuild && ((Player->getPower() * 0.75) < Player->getPowerUsed())){
			cost = p::uspool->getStructureTypeByName("APWR")->getCost();
			if (Player->getMoney() > cost){
				if (BuildStructure (Player, PlayerNumb, "APWR", ConstYardPos))
					Player->changeMoney(-1 * cost);
				StructureWasBuild = true;
			}
		}

		// Build the barracks
		if (!StructureWasBuild && (NumbBarracks == 0 && NumbConstYards > 0)){
			cost = p::uspool->getStructureTypeByName("BARR")->getCost();
			if (Player->getMoney() > cost){
				if (BuildStructure (Player, PlayerNumb, "BARR", ConstYardPos))
					Player->changeMoney(-1 * cost);
				StructureWasBuild = true;
			}
		}

		// Build ore tesla coils
		if (!StructureWasBuild && (NumbTeslaCoils < 2 && NumbConstYards > 0)){
			cost = p::uspool->getStructureTypeByName("TSLA")->getCost();
			if (Player->getMoney() > cost){
				if (BuildStructure (Player, PlayerNumb, "TSLA", ConstYardPos))
					Player->changeMoney(-1 * cost);
				StructureWasBuild = true;
			}
		}
	}

	if (NumbWarfactorys > 0){
		if (NumbOfOreTrucks < 1 || (NumbOfOreTrucks <3 && NumbTanks > 5)){
			// Get the structure type so we can see what the cost are for this structure
			UnitType* type = p::uspool->getUnitTypeByName("HARV");
			cost = type->getCost();
			if (Player->getMoney() > cost){
				if (p::dispatcher->unitSpawn(type, PlayerNumb))
					Player->changeMoney(-1 * cost);
			}else if (NumbOreRefs > 0 && NumbOfOreTrucks > 0)
				return;
		}else{
			// Build tanks
			// Get the structure type so we can see what the cost are for this structure
			UnitType* type = p::uspool->getUnitTypeByName("1TNK");
			cost = type->getCost();
			if (Player->getMoney() > cost){
				if (p::dispatcher->unitSpawn(type, PlayerNumb))
					Player->changeMoney(-1 * cost);
			}
		}


		// Build soldier
		// Get the structure type so we can see what the cost are for this structure
		if (NumbBarracks > 0){
			UnitType* type;
			switch (UnitBuildMultiplexer){
				case 0:
					type = p::uspool->getUnitTypeByName("E1");
					cost = type->getCost();
					if (Player->getMoney() > cost){
						if (p::dispatcher->unitSpawn(type, PlayerNumb))
							Player->changeMoney(-1 * cost);
					}
					break;
				case 1:
					type = p::uspool->getUnitTypeByName("E2");
					cost = type->getCost();
					if (Player->getMoney() > cost){
						if (p::dispatcher->unitSpawn(type, PlayerNumb))
							Player->changeMoney(-1 * cost);
					}
					break;
				case 2:
					type = p::uspool->getUnitTypeByName("E3");
					cost = type->getCost();
					if (Player->getMoney() > cost){
						if (p::dispatcher->unitSpawn(type, PlayerNumb))
							Player->changeMoney(-1 * cost);
					}
					break;
				case 3:
					type = p::uspool->getUnitTypeByName("E4");
					cost = type->getCost();
					if (Player->getMoney() > cost){
						if (p::dispatcher->unitSpawn(type, PlayerNumb))
							Player->changeMoney(-1 * cost);
					}
					break;
			}
			if (UnitBuildMultiplexer < 3)
				UnitBuildMultiplexer++;
			else
				UnitBuildMultiplexer = 0;
		}
	}
}
#if 1
// Assign tasks to the units
void ai::patrolAndAttack (Player *Player, int PlayerNumb)
{
int                         NumbUnits,
                            NumbStructures,
                            lPlayerNumbStructures;
std::vector<Structure*>     structurepool, lPlayerStructurePool;
std::vector<Unit*>          unitpool;
Unit                        *Unit;

	// don't try to control for the human player
	if (PlayerNumb == this->HumanPlayerNumb){
		return;
	}

	// Handle unit vars
	NumbUnits		= Player->getNumUnits();
	unitpool		= Player->getUnits();

	// Handle structure vars
	structurepool		= Player->getStructures();
	NumbStructures		= Player->getNumStructs();

	// Handle lPlayer vars
	lPlayerStructurePool	= p::ppool->getLPlayer()->getStructures();
	lPlayerNumbStructures	= p::ppool->getLPlayer()->getNumStructs();

	if (lPlayerNumbStructures > 0 && NumbUnits > 0){

		for (int UnitNumb = 0; UnitNumb < NumbUnits; UnitNumb++){
			Unit = unitpool[UnitNumb];
			if (!Unit->canAttack())
				continue;
			if (Unit->AI_Mission == AI_MISSION_NONE || Unit->AI_Mission == AI_MISSION_ATTACK_BASE){
				Unit->AI_Mission = AI_MISSION_ATTACK_BASE;
				for (int str =0; str < lPlayerNumbStructures; str++){
					if (lPlayerStructurePool[str]->isAlive()){
						Unit->attack(lPlayerStructurePool[str]);
						break;
					}
				}
			}
		}
	}
}
#else

// Assign tasks to the units
void ai::patrolAndAttack (Player *Player, int PlayerNumb)
{
int                         NumbUnits,
                            NumbStructures,
                            lPlayerNumbStructures;
std::vector<Structure*>     structurepool, lPlayerStructurePool;
std::vector<Unit*>          unitpool;
Unit                        *Unit;
int							LastUnit;
int							str = 0;

	// don't try to control for the human player
	if (PlayerNumb == this->HumanPlayerNumb){
		return;
	}

	// Handle unit vars
	NumbUnits		= Player->getNumUnits();
	unitpool		= Player->getUnits();

	// Handle structure vars
	structurepool		= Player->getStructures();
	NumbStructures		= Player->getNumStructs();

	// Handle lPlayer vars
	lPlayerStructurePool	= p::ppool->getLPlayer()->getStructures();
	lPlayerNumbStructures	= p::ppool->getLPlayer()->getNumStructs();

	if (lPlayerNumbStructures > 0 && NumbUnits > 0 ){
		// Attack the construction yard with half the units
		for (int UnitNumb = 0; UnitNumb < NumbUnits; UnitNumb++){
			Unit = unitpool[UnitNumb];
			if (!Unit->canAttack() /*|| Unit->IsAttacking ()*/)
				continue;
			if ((Unit->AI_Mission == AI_MISSION_NONE || Unit->AI_Mission == AI_MISSION_ATTACK_BASE) && !Unit->IsAttacking ()){
				Unit->AI_Mission = AI_MISSION_ATTACK_BASE;
				if (lPlayerStructurePool[str]->isAlive() ){
					Unit->attack(lPlayerStructurePool[str]);
					break;
				}
			}
			LastUnit = UnitNumb;
		}
#if 0
		LastUnit++;

		if (LastUnit >= NumbUnits)
			return;

		str = 1;
		if (str >= lPlayerNumbStructures)
			str = 0;

		// Attack other buildings with the other half of the units
		for (int UnitNumb = LastUnit; UnitNumb < NumbUnits; UnitNumb++){
			Unit = unitpool[UnitNumb];
			if (!Unit->canAttack() || Unit->IsAttacking ())
				continue;
			if (Unit->AI_Mission == AI_MISSION_NONE){
				Unit->AI_Mission = AI_MISSION_ATTACK_BASE;

				if (lPlayerStructurePool[str]->isAlive()){
					Unit->attack(lPlayerStructurePool[str]);
				}
				str++;
				if (str >= lPlayerNumbStructures)
					str = 0;
			}
		}
#endif
	}

}
#endif

// Assign tasks to the units
void ai::guideAttack (Player *Player, int PlayerNumb)
{
int                         NumbUnits,
                            NumbStructures,
                            lPlayerNumbStructures;
std::vector<Structure*>     structurepool, lPlayerStructurePool;
std::vector<Unit*>          unitpool;
Unit                        *Unit;
int							str = 0;

	// don't try to control for the human player
	if (PlayerNumb == this->HumanPlayerNumb){
		return;
	}

	// Handle unit vars
	NumbUnits		= Player->getNumUnits();
	unitpool		= Player->getUnits();

	// Handle structure vars
	structurepool		= Player->getStructures();
	NumbStructures		= Player->getNumStructs();

	// Handle lPlayer vars
	lPlayerStructurePool	= p::ppool->getLPlayer()->getStructures();
	lPlayerNumbStructures	= p::ppool->getLPlayer()->getNumStructs();

	if (lPlayerNumbStructures > 0 && NumbUnits > 0 ){
		// Attack the construction yard with half the units
		for (int UnitNumb = 0; UnitNumb < NumbUnits; UnitNumb++){
			Unit = unitpool[UnitNumb];
			if (!Unit->canAttack())
				continue;
			if (Unit->AI_Mission == AI_MISSION_ATTACK_BASE){
				if (lPlayerStructurePool[str]->isAlive() && !Unit->IsAttacking ()){
					Unit->attack(lPlayerStructurePool[str]);
					break;
				}
			}
			//LastUnit = UnitNumb;
		}
	}
}

//
//	For now the ai assumes that all non local players are computer players
//	This means no support for multiplayer with computer players mix.
//
void ai::handle (void)
{

Player 	*CurPlayer;


// Structures
//char			*Name;

	// Don't burn our fingers on multi player mode...
	if (ccmap->getGameMode() == GAME_MODE_MULTI_PLAYER){
		return;
	}

	switch (this->Difficulty){
		case DIFFICUTY_EASY:
//			if (AiDifficultyTimer.Elapsed () < 2)
//				return;
			break;

		case DIFFICUTY_MEDIUM:
//			if (AiDifficultyTimer.Elapsed () < 1)
//				return;
			break;

		case DIFFICUTY_HARD:
//			if (AiDifficultyTimer.Elapsed () < 0.5)
//				return;
			break;

		case DIFFICUTY_VERYHARD:
			// Don't wait
			break;
	}


//	c 	= p::ppool->getNumPlayers();
	this->HumanPlayerNumb	= p::ppool->getLPlayerNum();

	// This part of the AI takes care of the computer player attacking a enemy within range
	// and of the human player fighting back when attacked.
	for (int PlayerNumb = 0; PlayerNumb < this->NumbPlayers; PlayerNumb++){

		CurPlayer = p::ppool->getPlayer(PlayerNumb);

		// Let the units defend themselfes
		this->DefendUnits (CurPlayer,PlayerNumb);

		// Let the structures with weapons defend themselfes
		this->DefendStructures (CurPlayer,PlayerNumb);

		//The next part is only for computer players
		if (PlayerNumb == HumanPlayerNumb)
			continue;

		if ((SDL_GetTicks() - LastGuideTickCount[PlayerNumb]) > 1000){
			LastGuideTickCount[PlayerNumb] = SDL_GetTicks();
			this->guideAttack (CurPlayer, PlayerNumb);
		}

		if ((SDL_GetTicks() - LastBuildTickCount[PlayerNumb]) > 24000){
			LastBuildTickCount[PlayerNumb] = SDL_GetTicks();
			this->Build (CurPlayer, PlayerNumb);
		}

		if ((SDL_GetTicks() - LastAttackTickCount[PlayerNumb]) > 240000){
			LastAttackTickCount[PlayerNumb] = SDL_GetTicks();
			this->patrolAndAttack (CurPlayer, PlayerNumb);
		}
	}
}
