#include <cstring>
#include "actioneventqueue.h"
#include "common.h"
#include "config.h"
#include "playerpool.h"
#include "player.h"

PlayerPool::PlayerPool(INIFile *inifile, Uint8 gamemode)
{
    for (int i = 0; i < 8; i++)
        playerstarts[i] = 0;
    lost = false;
    won = false;
    mapini = inifile;
    updatesidebar = false;
    radarstatus = 0;
    this->gamemode = gamemode;
}

PlayerPool::~PlayerPool()
{
    Uint8 i;
    for( i = 0; i < playerpool.size(); i++ ) {
		if (playerpool[i] != NULL)
			delete playerpool[i];
		playerpool[i] = NULL;
    }
}

void PlayerPool::setLPlayer(const char* pname)
{
    Uint8 i;
    for( i = 0; i < playerpool.size(); i++ ) {
        if( !strcasecmp(playerpool[i]->getName(), pname) ) {
            localPlayer = i;
            return;
        }
    }
    //logger->warning("Tried to set local player to non-existing player \"%s\"\n", pname);
    playerpool.push_back(new Player(pname, mapini));
    localPlayer = static_cast<Uint8>(playerpool.size()-1);
    playerpool[localPlayer]->setPlayerNum(localPlayer);
}

void PlayerPool::setLPlayer(Uint8 number, const char* nick, const char* colour, const char* mside)
{
    Uint8 i;
    for( i = 0; i < playerpool.size(); i++ ) {
        if( playerpool[i]->getMSide() == number ) {
            localPlayer = i;
            playerpool[i]->setSettings(nick,colour,mside);
            return;
        }
    }
    //logger->warning("Tried to set local player to non-existing player number %i\n", number);
    /*  playerpool.push_back(new Player("multi", mapini, this));
        localPlayer = playerpool.size()-1;
        playerpool[localPlayer]->setSettings(nick,colour,mside);
    */
}

void PlayerPool::setPlayer(Uint8 number, const char* nick, const int colour, const char* mside)
{
    Uint8 i;
    for( i = 0; i < playerpool.size(); i++ ) {
        if( playerpool[i]->getMSide() == number ) {
            localPlayer = i;
            playerpool[i]->setSettings(nick,colour,mside);
            return;
        }
    }
    //logger->warning("Tried to set local player to non-existing player number %i\n", number);
    /*  playerpool.push_back(new Player("multi", mapini, this));
        localPlayer = playerpool.size()-1;
        playerpool[localPlayer]->setSettings(nick,colour,mside);
    */
}

Uint8 PlayerPool::getPlayerNum(const char *pname)
{
    Uint8 i;
    for( i = 0; i < playerpool.size(); i++ ) {
        if( !strcasecmp(playerpool[i]->getName(), pname) ) {
            return i;
        }
    }
//	printf ("%s line %i: Create new player: %s\n", __FILE__, __LINE__, pname);
    playerpool.push_back(new Player(pname, mapini));
    playerpool[playerpool.size()-1]->setPlayerNum(static_cast<Uint8>(playerpool.size()-1));
    return static_cast<Uint8>(playerpool.size()-1);
}

char RA_house[20][10] = {"Spain", "Greece", "Ussr", "England", "Ukraine", "Germany", "France", "Turkey", "Goodguy", "Badguy", "Special", "Neutral", "Multi1", "Multi2", "Multi3" ,"Multi4", "Multi5", "Multi6", "Multi7", "Multi8"};

int PlayerPool::getPlayerNumByHouseNum(Uint8 House)
{
Uint8 i;
    if (House > 19)
        return -1;

    for( i = 0; i < playerpool.size(); i++ ) {
        if( !strcasecmp(playerpool[i]->getName(), RA_house[House]) ) {
            return i;
        }
    }
    return -1;
}

int PlayerPool::getHouseNumByPlayerNum(Uint8 Player)
{
Uint8 i;
    if (Player >= playerpool.size())
        return -1;

    for( i = 0; i < 19; i++ ) {
        if( !strcasecmp(playerpool[Player]->getName(), RA_house[i]) ) {
            return i;
        }
    }
    return -1;
}

Player* PlayerPool::getPlayerByName(const char* pname)
{
    return playerpool[getPlayerNum(pname)];
}

std::vector<Player*> PlayerPool::getOpponents(Player* pl)
{
    std::vector<Player*> opps;
    for(Uint8 i = 0; i < playerpool.size(); i++ ) {
        if (!playerpool[i]->isDefeated() && !pl->isAllied(playerpool[i])) {
            opps.push_back(playerpool[i]);
        }
    }
    return opps;
}

void PlayerPool::playerDefeated(Player *pl)
{
    Uint8 i;

    pl->clearAlliances();
    for( i = 0; i < playerpool.size(); i++ ) {
        if( playerpool[i] == pl ) {
            break;
        }
    }
    if( i == localPlayer ) {
        lost = true;
    }
    if (!lost) {
        Uint8 defeated = 0;
        for (i = 0; i < playerpool.size(); ++i) {
            if (playerpool[i]->isDefeated()) {
                ++defeated;
            }
        }
        if (playerpool.size() - defeated == 1) {
            won = true;
        } else if (playerpool.size() - defeated == playerpool[localPlayer]->getNumAllies() ) {
            won = true;
        }
    }
}

void PlayerPool::playerUndefeated(Player* pl)
{
    Uint8 i;

    pl->setAlliances();
    for( i = 0; i < playerpool.size(); i++ ) {
        if( playerpool[i] == pl ) {
            break;
        }
    }
    if( i == localPlayer ) {
        lost = false;
    }
    if (gamemode == 0) {
        if (!lost) {
            won = false;
        }
    }
}

INIFile *PlayerPool::getMapINI()
{
    return mapini;
}

void PlayerPool::setAlliances()
{
    for (Uint16 i=0; i < playerpool.size() ; ++i) {
        playerpool[i]->setAlliances();
    }
}

void PlayerPool::placeMultiUnits()
{
    for (Uint16 i=0; i < playerpool.size() ; ++i) {
        if (playerpool[i]->getPlayerStart() != 0) {
            playerpool[i]->placeMultiUnits();
        }else
			printf ("%s line %i: Failed to get player start\n", __FILE__, __LINE__);
    }
}

Uint16 PlayerPool::getAStart()
{
    Uint8 rnd,sze = 0;
    Uint16 rv;

    for (int i = 0; i < 8; i++){
        if (playerstarts[i] != 0)
            sze++;
    }

    // pick a starting location at random
    rnd = (int) ((double)sze*rand()/(RAND_MAX+1.0));
    while (playerstarts[rnd] == 0) {
        rnd = (int) ((double)sze*rand()/(RAND_MAX+1.0));
    }
    rv  = playerstarts[rnd];
    // ensure this starting location is not reused.
    playerstarts[rnd] = 0;

    return rv;
}

void PlayerPool::setPlayerStarts(Uint8 pos, Uint32 start)
{
    if (pos < 8)
        playerstarts[pos] = start;
}

bool PlayerPool::pollSidebar()
{
    if (updatesidebar) {
        updatesidebar = false;
        return true;
    }
    return false;
}

void PlayerPool::updateSidebar()
{
    updatesidebar = true;
}

Uint8 PlayerPool::statRadar()
{
    Uint8 tmp = radarstatus;
    radarstatus = 0;
    return tmp;
}

void PlayerPool::updateRadar(Uint8 status)
{
    radarstatus = status;
}
