#include <cstdlib>
#include <cstring>
#include "ccmap.h"
#include "common.h"
#include "inifile.h"
#include "logger.h"
#include "playerpool.h"
#include "shpimage.h"
#include "soundengine.h"
#include "structure.h"
#include "unit.h"
#include "unitandstructurepool.h"
#include "weaponspool.h"
#include "dispatcher.h"

StructureType::StructureType(const char* typeName, INIFile *structini,
        INIFile *artini, const char* thext) : UnitOrStructureType()
{
    SHPImage *shpimage, *makeimage;
    Uint32 i;
    char shpname[13], imagename[8], *tmpCharges, *tmpWaterBound;
    char* tmp;
    char blocktest[128];
    Uint32 size;
    char* miscnames;

    // Ensure that there is a section in the ini file
    try {
        structini->readKeyValue(typeName,0);
    } catch (int) {
        shpnums = NULL;
        blocked = NULL;
        shptnum = NULL;
        name    = NULL;
        return;
    }

    is_wall = false;

    memset(this->tname,0x0,8);
    strncpy(this->tname,typeName,8);
    name = structini->readString(tname,"name");
    //prereqs = structini->splitList(tname,"prerequisites",',');
    tmp = structini->readString(tname, "prerequisites");
//	printf ("%s line %i: Structure: %s --> prereqs = %s\n", __FILE__, __LINE__, tname, tmp);

    if( tmp != NULL ) {
        prereqs = splitList(tmp, ',');
        delete[] tmp;
		tmp = NULL;
    }
#if 0
	printf ("%s line %i: Structure: %s \n", __FILE__, __LINE__, tname);
	for (unsigned int j = 0; j < prereqs.size(); j++){
		printf ("prereq = %s\n", prereqs[j]);
	}
#endif
    tmp = structini->readString(tname, "deploywith");

    if( tmp != NULL ) {
        deploywith = splitList(tmp, ',');
        delete[] tmp;
		tmp = NULL;
    }

    //owners = structini->splitList(tname,"owners",',');
    tmp = structini->readString(tname, "owners");
    if(  tmp != NULL ) {
        owners = splitList(tmp, ',');
        delete[] tmp;
		tmp = NULL;
    }
    if (owners.empty()) {
        logger->warning("%s has no owners\n",tname);
    }

    if( !strcasecmp(tname, "BRIK") || !strcasecmp(tname, "SBAG") ||
            !strcasecmp(tname, "BARB") || !strcasecmp(tname, "WOOD") ||
            !strcasecmp(tname, "CYCL") || !strcasecmp(tname, "FENC") )
        is_wall = true;

    /* the size of the structure in tiles */
    xsize = artini->readInt(tname, "xsize",1);
    ysize = artini->readInt(tname, "ysize",1);

    blckoff = 0;
    blocked = new Uint8[xsize*ysize];
    for( i = 0; i < (Uint32)xsize*ysize; i++ ) {
        sprintf(blocktest, "notblocked%d", i);
        size = artini->readInt(tname, blocktest);
        if(size == INIERROR) {
            blocked[i] = 1;
            xoffset = -(i%xsize)*24;
            yoffset = -(i/xsize)*24;
            if (blocked[blckoff] == 0) {
                blckoff = i;
            }
        } else {
            blocked[i] = 0;
        }
    }

    memset(shpname,0,13);
    numshps = structini->readInt(tname, "layers",1);

    shpnums = new Uint16[(is_wall?numshps:numshps+1)];
    shptnum = new Uint16[numshps];

    buildlevel = structini->readInt(tname,"buildlevel",100);
    techlevel = structini->readInt(tname,"techlevel",99);
    if (buildlevel == 100) {
        logger->warning("%s does not have a buildlevel\n",tname);
    }

    powerinfo.power = structini->readInt(tname,"power",0);
    powerinfo.drain = structini->readInt(tname,"drain",0);
    powerinfo.powered = structini->readInt(tname,"powered",0) != 0;
    maxhealth = structini->readInt(tname,"health",100);

    tmpCharges = (structini->readString(tname, "Charges"));
    charges = false;
    if (tmpCharges != NULL){
        if (strcmp (tmpCharges, "yes") == 0){
            charges = true;
        }
        delete[] tmpCharges;
		tmpCharges = NULL;
    }

    tmpWaterBound = (structini->readString(tname, "WaterBound"));
    WaterBound = false;
    if (tmpWaterBound != NULL){
        if (strcmp (tmpWaterBound, "yes") == 0){
            WaterBound = true;
        }
        delete[] tmpWaterBound;
		tmpWaterBound = NULL;
    }


    for( i = 0; i < numshps; i++ ) {
        sprintf(imagename, "image%d", i+1);
        tmp = structini->readString(tname, imagename);
        if( tmp == NULL ) {
            strncpy(shpname, tname, 13);
            strncat(shpname, ".SHP", 13);
        } else {
            strncpy(shpname, tmp, 13);
            delete[] tmp;
			tmp = NULL;
        }
        try {
            shpimage = new SHPImage(shpname, mapscaleq);
        } catch (ImageNotFound&) {
            strncpy(shpname, tname, 13);
            strncat(shpname, thext, 13);
            try {
                shpimage = new SHPImage(shpname, mapscaleq);
            } catch (ImageNotFound&) {
                logger->warning("Image not found: \"%s\"\n", shpname);
                numshps = 0;
                return;
            }
        }
        shpnums[i] = pc::imagepool->size();
        shptnum[i] = shpimage->getNumImg();
        pc::imagepool->push_back(shpimage);
    }
    if (!is_wall) {
        numwalllevels = 0;
        animinfo.loopend = structini->readInt(tname,"loopend",0);
        animinfo.loopend2 = structini->readInt(tname,"loopend2",0);

        animinfo.animspeed = structini->readInt(tname,"animspeed", 3);
        animinfo.animspeed = abs(animinfo.animspeed);
        animinfo.animspeed = (animinfo.animspeed>1?animinfo.animspeed:2);
        animinfo.animdelay = structini->readInt(tname,"delay",0);

        animinfo.animtype = structini->readInt(tname, "animtype", 0);
        animinfo.sectype  = structini->readInt(tname, "sectype", 0);

        animinfo.dmgoff   = structini->readInt(tname, "dmgoff", ((shptnum[0]-1)>>1));
        if (numshps == 2)
            animinfo.dmgoff2 = structini->readInt(tname, "dmgoff2", (shptnum[1]>>1));
        else
            animinfo.dmgoff2 = 0;

        defaultface       = structini->readInt(tname, "defaultface", 0);

        if (strlen(tname) <= 4) {
            strncpy(shpname, tname, 13);
            strncat(shpname, "make.shp", 13);
        } else
            logger->warning("%s is nonstandard!\n",tname);
        try {
            makeimage = new SHPImage(shpname, mapscaleq);
            makeimg = pc::imagepool->size();
            animinfo.makenum = makeimage->getNumImg();
            pc::imagepool->push_back(makeimage);
        } catch (ImageNotFound&) {
            makeimg = 0;
            animinfo.makenum = 0;
        }

        miscnames = structini->readString(tname, "primary_weapon");
        if( miscnames == NULL ) {
            primary_weapon = NULL;
        } else {
            primary_weapon = p::weappool->getWeapon(miscnames);
            delete[] miscnames;
			miscnames = NULL;
        }
        miscnames = structini->readString(tname, "secondary_weapon");
        if( miscnames == NULL ) {
            secondary_weapon = NULL;
        } else {
            secondary_weapon = p::weappool->getWeapon(miscnames);
            delete[] miscnames;
			miscnames = NULL;
        }
        turret = (structini->readInt(tname,"turret",0) != 0);
        if (turret) {
            turnspeed = structini->readInt(tname,"rot",3);
        }
    } else {
        numwalllevels = structini->readInt(tname,"levels",1);
        turret = 0;
        primary_weapon = NULL;
        secondary_weapon = NULL;
    }
    cost = structini->readInt(tname, "cost", 0);
    if (0 == cost) {
        logger->error("\"%s\" has no cost, resetting to 1\n", tname);
        cost = 1;
    }
    miscnames = structini->readString(tname,"armour","none");
    if (miscnames == NULL)
        armour = AC_none;
    else {
        if (strncasecmp(miscnames,"none",4) == 0)
            armour = AC_none;
        else if (strncasecmp(miscnames,"wood",4) == 0)
            armour = AC_wood;
        else if (strncasecmp(miscnames,"light",5) == 0)
            armour = AC_light;
        else if (strncasecmp(miscnames,"heavy",5) == 0)
            armour = AC_heavy;
        else if (strncasecmp(miscnames,"concrete",8) == 0)
            armour = AC_concrete;
        delete[] miscnames;
		miscnames = NULL;
    }

    primarysettable = (structini->readInt(tname,"primary",0) != 0);
    valid = true;
}


StructureType::~StructureType()
{
    Uint16 i;
    for (i=0;i<owners.size();++i){
		if (owners[i] != NULL)
			delete[] owners[i];
		owners[i] = NULL;
	}
    for (i=0;i<prereqs.size();++i){
		if (prereqs[i] != NULL)
			delete[] prereqs[i];
		prereqs[i] = NULL;
	}
	if (shpnums != NULL)
		delete[] shpnums;
	shpnums = NULL;
	if (blocked != NULL)
		delete[] blocked;
	blocked = NULL;
	if (shptnum != NULL)
		delete[] shptnum;
	shptnum = NULL;
	if (name != NULL)
		delete[] name;
	name = NULL;
}

Structure::Structure(StructureType *type, Uint16 cellpos, Uint8 owner,
        Uint16 rhealth, Uint8 facing, string trigger_name) : UnitOrStructure()
{
    Uint32 i;
    targetcell = cellpos;
    this->type = type;
    imagenumbers = new Uint16[type->getNumLayers()];
    if (!type->hasTurret()) {
        facing = 0;
    }
    for(i=0;i<type->getNumLayers();i++) {
        imagenumbers[i] = facing;
        if( owner != 0xff && !type->isWall() ) {
            imagenumbers[i] |= (p::ppool->getStructpalNum(owner)<<11);
        }
    }
    exploding = false;
    this->owner = owner;
    this->cellpos = cellpos;
    bcellpos = cellpos+(type->getBlckOff() % type->getXsize()) +
               ((type->getBlckOff()/type->getXsize())*p::ccmap->getWidth());
    animating = false;
    usemakeimgs = false;
    primary = false;
    buildAnim = NULL;
    attackAnim = NULL;
    health = (Uint16)((double)rhealth/256.0f * (double)type->getMaxHealth());
    damaged = checkdamage();
    if( !type->isWall() ) {
//	printf ("%s line %i: Build structure\n", __FILE__, __LINE__ );
        p::ppool->getPlayer(owner)->builtStruct(this);
    }

    // Handle deploy with...
    for (unsigned int j = 0; j < type->getDeployWith().size(); j++){
    //
	Uint16 x = cellpos%p::ccmap->getWidth();
	Uint16 y = cellpos/p::ccmap->getWidth();
	y += 2;
	int temppos = p::ccmap->translateToPos(x, y);
	while ( !p::ccmap->isBuildableAt(this->getOwner(), temppos) && y < p::ccmap->getHeight() ){
		y += 1;
		temppos = p::ccmap->translateToPos(x, y);
//		printf ("x = %i, y = %i, temppos = %i, Mapwidth = %i\n", x, y, temppos, p::ccmap->getWidth());
	}

	if (p::ccmap->isBuildableAt(this->getOwner(), temppos)){
		// Health = 256 --> max
		p::uspool->createUnit(this->type->getDeployWith()[j]/*"HARV"*/, temppos, 0, owner, 256, 0);
	}

	// If the current structure is a PROC and we are placing a extra harvester
       if (strcmp ((char*)this->type->getDeployWith()[j], "HARV") == 0 && 
           strcmp ((char*)type->getTName(), "PROC") == 0) {

		//	Set base refinary
		if (p::uspool->getUnitAt(temppos, 0) != NULL)
			p::uspool->getUnitAt(temppos, 0)->SetBaseRefinery (this);	
	}


    }

//	sec_anim = NULL;

	retry_sell = false;
	retry_repair = false;
	repairing = false;

    TriggerName = trigger_name;
}

Structure::~Structure()
{
	logger->warning ("%s line %i: Structure destructor\n", __FILE__, __LINE__);

	if (imagenumbers != NULL)
		delete[] imagenumbers;
	imagenumbers = NULL;
}

Uint16 Structure::getBPos(Uint16 pos) const
{
    Uint16 x,y,dx,dy,t,retpos,bpos,sc;
    Sint16 dw;
    Uint32 mwid = p::ccmap->getWidth();
    x = cellpos%mwid;
    dx = 0;
    if ((pos%mwid) > x) {
        dx = min((unsigned)(type->getXsize()-1),(unsigned)((pos%mwid)-x)-1);
    }
    y = cellpos/mwid;
    dy = 0;
    if ((pos/mwid) > y) {
        dy = min((unsigned)(type->getYsize()-1),(unsigned)((pos/mwid)-y)-1);
    }
    retpos = (x+dx)+(y+dy)*mwid;
    // just makes the bpos calculation cleaner
    sc = x+y*mwid;
    dw = type->getXsize() - mwid;
    bpos   = retpos - sc + dy*dw;
    while (!type->isBlocked(dx+dy*type->getXsize())) {
        /* This happens in this situation (P is position of attacker,
         * X is a blocked cell and _ is an unblocked cell)
         * P   P
         *  _X_
         *  XXX
         *  _X_
         * P   P
         */
        if (dx == type->getXsize()-1) {
            for (t=dx;t>0;--t) {
                retpos = (x+t)+(y+dy)*mwid;
                bpos   = retpos - sc + dy*dw;
                if (type->isBlocked(bpos)) {
                    return retpos;
                }
            }
        } else {
            for (t=dx;t<type->getXsize();++t) {
                retpos = (x+t)+(y+dy)*mwid;
                bpos   = retpos - sc + dy*dw;
                if (type->isBlocked(bpos)) {
                    return retpos;
                }
            }
        }
        ++dy;
        if (dy >= type->getYsize()) {
            logger->error("ERROR: could not find anywhere to shoot at %s!\n",type->getTName());
        }
        retpos = (x+dx)+(y+dy)*mwid;
    }
    return retpos;
}

/// Helper function for getFreePos
/// @BUG Doesn't check that the terrain is passable (only buildable).
static bool valid_pos(StructureType *type, Uint8 PlayerNr, Uint16 pos, Uint8*) {
    return p::ccmap->isBuildableAt( PlayerNr, pos, type->isWaterBound());
}

/// Helper function for getFreePos
/// @BUG Doesn't check that the terrain is passable (only buildable).
static bool valid_possubpos(StructureType *type, Uint8 PlayerNr, Uint16 pos, Uint8* subpos) {
    InfantryGroup *ig = p::uspool->getInfantryGroupAt(pos);
    if (ig == NULL) {
        return p::ccmap->isBuildableAt( PlayerNr, pos, type->isWaterBound());
    }else{
	    if (ig->IsAvailable()) {
    	    *subpos = ig->GetFreePos();
        	return true;
    	}
	}
    return false;
}

/// @BUG This literally doesn't handle edge cases.
Uint16 Structure::getFreePos(Uint8* subpos, bool findsubpos) {
    bool (*checker)(StructureType*, Uint8, Uint16, Uint8*);
    Uint8 i, xsize, ysize;
    Uint16 x, y, curpos;

    xsize = type->getXsize();
    ysize = type->getYsize();

    curpos = cellpos;
    p::ccmap->translateFromPos(curpos, &x, &y);
    y += ysize; // bottom left of building
    curpos = p::ccmap->translateToPos(x,y);

    if (findsubpos) {
        checker = valid_possubpos;
    } else {
        checker = valid_pos;
    }
    if (!checker(type, this->getOwner(), curpos, subpos))
        curpos = 0xffff;

    for (i = 0; (i < xsize && 0xffff == curpos); ++i) {
        ++x;
        curpos = p::ccmap->translateToPos(x, y);
        if (!checker(type, this->getOwner(), curpos, subpos))
            curpos = 0xffff;
    }
    // ugly: I assume that the first blocks are noblocked - g0th
    for (i = 0;(i < ysize && 0xffff == curpos); ++i) {
        --y;
        curpos = p::ccmap->translateToPos(x, y);
        if (!checker(type, this->getOwner(), curpos, subpos))
            curpos = 0xffff;
    }
    for (i = 0; (i < (xsize+1) && 0xffff == curpos); ++i) {
        --x;
        curpos = p::ccmap->translateToPos(x, y);
        if (!checker(type, this->getOwner(), curpos, subpos))
            curpos=0xffff;
    }
    for (i = 0; (i < ysize && 0xffff == curpos); ++i) {
        ++y;
        curpos = p::ccmap->translateToPos(x, y);
        if (!checker(type, this->getOwner(), curpos, subpos))
            curpos=0xffff;
    }
    return curpos;
}

void Structure::remove() {
    if (!type->isWall()) {
        p::ppool->getPlayer(owner)->lostStruct(this);
    }
    UnitOrStructure::remove();
}

void Structure::repairDone (void)
{
	repairing = false;
}

void Structure::sell() 
{
	printf ("%s line %i: Start sell animation\n", __FILE__, __LINE__);
	runAnim(9);
}

void Structure::repair() 
{
	printf ("%s line %i: Start repair animation\n", __FILE__, __LINE__);
	runAnim(8);
	repairing = true;
//	runSecAnim (8);
}

bool Structure::isRepairing ()
{
	return repairing;
}

/** Method to get a list of imagenumbers which the renderer will draw. */
Uint8 Structure::getImageNums(Uint32 **inums, Sint8 **xoffsets, Sint8 **yoffsets) {
    Uint16 *shps;
    int i;

	if (retry_sell)
		runAnim(9);

	if (retry_repair)
		runAnim(8);

	shps = type->getSHPNums();

//	printf ("Get image, imagenumbers[0] = %i\n", imagenumbers[0]);

	if (usemakeimgs && (!type->isWall()) && (type->getMakeImg() != 0)) {
		*inums = new Uint32[1];
		*xoffsets = new Sint8[1];
		*yoffsets = new Sint8[1];
		(*inums)[0] = (type->getMakeImg()<<16)|imagenumbers[0];
		(*xoffsets)[0] = type->getXoffset();
		(*yoffsets)[0] = type->getYoffset();
		return 1;
	} else {
		*inums = new Uint32[type->getNumLayers()];
		*xoffsets = new Sint8[type->getNumLayers()];
		*yoffsets = new Sint8[type->getNumLayers()];
		for(i = 0; i < type->getNumLayers(); i++ ) {
			(*inums)[i] = (shps[i]<<16)|imagenumbers[i];
			(*xoffsets)[i] = type->getXoffset();
			(*yoffsets)[i] = type->getYoffset();
		}
		return type->getNumLayers();
	}
}

void Structure::runAnim(Uint32 mode)
{
Uint32 speed;

//	logger->note ("%s line %i: DEBUG: run anim mode %i\n",__FILE__, __LINE__, mode);


	// If we want to sell stop current animations
	if (mode == 9 && animating ){
		this->stopAnim();
		retry_sell = true;
		return;
	}
#if 0
	// If we want to repair stop current animations
	if (mode == 8 && animating ){
		this->stopAnim();
		retry_repair = true;
	}
#endif

	if (!animating) {
		animating = true;
		if (mode == 0) { // run build anim at const speed
			usemakeimgs = true;
			buildAnim = new BuildAnimEvent(3,this,false);
		} else if (mode == 9) { // run sell anim at const speed
			usemakeimgs = true;
			// Set the image number to the correct value for using make images ;)
			if (getNumbImages( 0 ) > type->getAnimInfo().makenum - 1)
				setImageNum(type->getAnimInfo().makenum - 1, 0);
			else
				logger->error ("%s line %i: Failed to set frame %i layer %i, numb frames = %i\n", __FILE__, __LINE__, type->getAnimInfo().makenum - 1, 0, getNumbImages( 0 ));
			buildAnim = new BuildAnimEvent(3,this,true);
			retry_sell = false;
		}  else if (mode == 8) { // run repair anim at const speed
			buildAnim = new RepairAnimEvent(3,this);
			retry_repair = false;
		}else {
			speed = type->getAnimInfo().animspeed;
			switch (mode&0xf) {
				case 1:
					if (type->getAnimInfo().animtype == 4) {
//						logger->note ("%s line %i: DEBUG: run anim mode 1 (proc anim event)\n",__FILE__, __LINE__);
						buildAnim = new ProcAnimEvent(speed,this);
					} else {
//						logger->note ("%s line %i: DEBUG: run anim mode 1 (loop anim event)\n",__FILE__, __LINE__);
						buildAnim = new LoopAnimEvent(speed,this);
					}
					break;
				case 2:
//					logger->note ("%s line %i: DEBUG: run anim mode 2 (turn anim event)\n",__FILE__, __LINE__);
					buildAnim = new BTurnAnimEvent(speed,this,(mode>>4));
					break;
				case 7:
//					logger->note ("%s line %i: DEBUG: run anim mode 7 (refine anim event)\n",__FILE__, __LINE__);
					buildAnim = new RefineAnimEvent(speed,this,5);
					break;
				default:
					buildAnim = NULL;
					animating = false;
					break;
			}
		}
		if (buildAnim != NULL) {
			p::aequeue->scheduleEvent(buildAnim);
		}
	}
	//else
	//	logger->note ("%s line %i: Warning: already animating\n", __FILE__, __LINE__);
}

void Structure::runSecAnim(Uint32 param)
{
    BuildingAnimEvent* sec_anim = NULL;
    Uint8 secmode = type->getAnimInfo().sectype;
    if (secmode != 0) {
        switch (secmode) {
        case 7:
            sec_anim = new RefineAnimEvent(2,this,param);
            break;
        case 5:
            sec_anim = new DoorAnimEvent(2,this,(param!=0));
            break;
//        case 8:
//            sec_anim = new RepairAnimEvent(3,this);
//            break;
        }
        if (animating) {
            buildAnim->setSchedule(sec_anim);
            stopAnim();
        } else {
            buildAnim = sec_anim;
            p::aequeue->scheduleEvent(buildAnim);
            animating = true;
        }
    }
}

void Structure::stopAnim()
{
    buildAnim->stop();
}

void Structure::stop()
{
    if (attackAnim != NULL) {
		attackAnim->stop();
    }
}


void Structure::applyDamage(Sint16 amount, Weapon* weap, UnitOrStructure* attacker)
{
	if (exploding)
		return;

//	logger->note ("%s line %i: Update damage\n", __FILE__, __LINE__);


	if (this->getOwner() != p::ppool->getLPlayerNum()){
		// This structure is from a computer player
		if (p::ccmap->getGameMode() != GAME_MODE_SINGLE_PLAYER){
			pc::Ai->DefendComputerPlayerBaseUnderAttack ( p::ppool->getPlayer(this->getOwner()), this->getOwner(), attacker, this );
		}
	}else {
		// The structure is from the local player
		if ((SDL_GetTicks() - LastAttackTick) > (60 * 1000)){
			pc::sfxeng->PlaySound("baseatk1.aud");
			LastAttackTick = SDL_GetTicks();
		}
	}


	Uint8 odam = damaged;

	if (weap != NULL)
		amount = (Sint16)((double)amount * weap->getVersus(type->getArmour()));

	if ((health-amount) <= 0) {

		exploding = true;

		HandleTriggers ( (UnitOrStructure*)this, TRIGGER_EVENT_DESTROYED );
		HandleTriggers ( (UnitOrStructure*)this, TRIGGER_EVENT_ATTACKED );

		// Play building destroyed sound
		if (pc::sfxeng != NULL) {
			if (pc::Config.gamenum == GAME_TD ) {
				pc::sfxeng->PlaySound("xplobig4.aud");
			}else if (pc::Config.gamenum == GAME_RA ){
				pc::sfxeng->PlaySound("kaboom22.aud");
			}
		}

		if (type->isWall()) {
			p::uspool->removeStructure(this);
		} else {
			if (attacker != NULL)
				p::ppool->getPlayer(attacker->getOwner())->addStructureKill();

			//printf ("%s line %i: Start explode anim for structure: %i\n", __FILE__, __LINE__, (int) this);
			BExplodeAnimEvent* boom = new BExplodeAnimEvent(1,this);
			if (animating) {
				buildAnim->setSchedule(boom);
				buildAnim->stop();
			} else {
				buildAnim = boom;
				p::aequeue->scheduleEvent(boom);
			}
		}
		return;
	} else if ((health-amount)>type->getMaxHealth()) {
		health = type->getMaxHealth();
	} else {
		health -= amount;
	}
	HandleTriggers ( (UnitOrStructure*)this, TRIGGER_EVENT_ATTACKED );

    if (animating) {
        buildAnim->updateDamaged();
    } else {
        if (type->isWall()) {
            if ((damaged = checkdamage())) { // This is correct
                if (odam != damaged) {
                    changeImage(0,16);
                }
            }
        } else {
            if ((damaged = checkdamage())) { // This is correct
				Uint32 ImageNr = (unsigned)(imagenumbers[0]&~0x800);
                if (odam != damaged) { // only play critical damage sound once
                    if (pc::sfxeng != NULL) {
						if (pc::Config.gamenum == GAME_TD ) {
							pc::sfxeng->PlaySound("xplobig4.aud");
						}else if (pc::Config.gamenum == GAME_RA ){
							pc::sfxeng->PlaySound("kaboom1.aud");
						}
                    }
					if (getNumbImages( 0 ) > (unsigned)(imagenumbers[0]&0x7FF) + (unsigned)type->getAnimInfo().dmgoff)
                    	setImageNum((unsigned)(imagenumbers[0]&0x7FF)+ (unsigned)type->getAnimInfo().dmgoff,0);
					else
						logger->error ("%s line %i: FAILED imagenr = %u, type->getAnimInfo().dmgoff = %u, ImgNr == %u\n", __FILE__, __LINE__, (unsigned)(imagenumbers[0]&0x7FF), (unsigned)type->getAnimInfo().dmgoff, ImageNr);
                    if (type->getNumLayers() == 2){
						if (getNumbImages( 1 ) > (unsigned)(imagenumbers[1]&0x7FF) + (unsigned)type->getAnimInfo().dmgoff2)
							setImageNum((unsigned)(imagenumbers[1]&0x7FF)+ (unsigned)type->getAnimInfo().dmgoff2,1);
						else
							logger->error ("%s line %i: Failed to set frame %i layer %i, numb frames = %i\n", __FILE__, __LINE__, (imagenumbers[1]&~0x800)+type->getAnimInfo().dmgoff2, 1, getNumbImages( 1 ));
					}
                }
	} else {
                if (odam) {
			if (getNumbImages( 0 ) > (unsigned)(imagenumbers[0]&0x7FF) + (unsigned)type->getAnimInfo().dmgoff)
				setImageNum((unsigned)(imagenumbers[0]&0x7FF) + (unsigned)type->getAnimInfo().dmgoff,0);
			else
				logger->error ("%s line %i: Failed to set frame %i layer %i, numb frames = %i\n", __FILE__, __LINE__, (imagenumbers[0]&~0x800)-type->getAnimInfo().dmgoff, 0, getNumbImages( 0 ));
			if (type->getNumLayers() == 2){
				if (getNumbImages( 1 ) > (unsigned)(imagenumbers[1]&0x7FF) + (unsigned)type->getAnimInfo().dmgoff)
					setImageNum((unsigned)(imagenumbers[1]&0x7FF) + (unsigned)type->getAnimInfo().dmgoff,1);
				else
					logger->error ("%s line %i: Failed to set frame %i layer %i, numb frames = %i\n", __FILE__, __LINE__, (imagenumbers[1]&~0x800)-type->getAnimInfo().dmgoff, 1, getNumbImages( 1 ));
			}
			return;
                }
            }
        }
    }
}

void Structure::ChangeHealth (Sint16 amount)
{
//	printf ("%s line %i: original health = %i\n", __FILE__, __LINE__, health);
	if (health + amount > type->getMaxHealth()){
		health = type->getMaxHealth();
	}else if (health + amount < 0){
		health = 0;
	}else
		health += amount;

//	printf ("%s line %i: health = %i\n", __FILE__, __LINE__, health);
}

bool Structure::IsAttacking (void)
{
    if (attackAnim == NULL)
        return false;
    return true;
}

void Structure::attack(UnitOrStructure* target)
{
    // Don't attack while the structure is being created
	if (usemakeimgs)
		return;

	// Only attack if we have a weapon
	if (!this->canAttack())
		return;
#if 0
	// Check that we have power enough to attack
	if ( type->getPowerInfo().powered){
		if ( p::ppool->getPlayer(this->getOwner())->getPower() < p::ppool->getPlayer(this->getOwner())->getPowerUsed()){
			return;
		}
	}

	// printf ("%s line %i: PowerNeeded = %i, Power = %i, Player = %i\n", __FILE__, __LINE__, p::ppool->getPlayer(this->getOwner())->getPowerUsed(), p::ppool->getPlayer(this->getOwner())->getPower(), this->getOwner());
#endif
    this->target = target;
    targetcell = target->getBPos(cellpos);
    if( attackAnim == NULL ) {
        attackAnim = new BAttackAnimEvent(0, this);
        p::aequeue->scheduleEvent(attackAnim);
    } else {
        attackAnim->update();
    }
}

Uint8 Structure::checkdamage()
{
    ratio = ((double)health)/((double)type->getMaxHealth());
    if (type->isWall()) {
        if ((ratio <= 0.33)&&(type->getNumWallLevels() == 3))
            return 2;
        else
            return ((ratio <= 0.66)&&(type->getNumWallLevels() > 1));
    } else {
        return (ratio <= 0.5);
    }
}

Uint16 Structure::getNumbImages( Uint8 layer )
{
	if (!usemakeimgs){
		if (type->getSHPTNum() != NULL)
			return type->getSHPTNum()[layer];
	}else{
		return type->getAnimInfo().makenum; 
	}
	return 0;
}


void Structure::setImageNum(Uint32 num, Uint8 layer)
{
//	if (strcmp ((char*)type->getTName(), "PROC") == 0) 
//		printf ("%s line %i: Set image numb %i, layer = %i\n", __FILE__, __LINE__, num, layer);
	if (getNumbImages( layer ) > num)
		imagenumbers[layer]=(num)|(p::ppool->getStructpalNum(owner)<<11);
	else
		logger->error ("%s line %i: Failed to set frame %i layer %i, numb frames = %i\n", __FILE__, __LINE__, num, layer, getNumbImages( layer ));
}

// This should be customisable somehow
Uint32 Structure::getExitCell() const
{
    return cellpos+(type->getYsize()*p::ccmap->getWidth());
}

Uint16 Structure::getTargetCell() const
{
    if (attackAnim != NULL && target != NULL) {
        return target->getBPos(cellpos);
    }
    return targetcell;
}

