// mode: -*- C++ -*-
/*****************************************************************************
 * graphicsengine.h - header file for the GraphicsEngine class
 *
 ****************************************************************************/
#ifndef GRAPHICSENGINE_H
#define GRAPHICSENGINE_H

#include "SDL.h"
#include <vector>
#include "message.h"
#include "playerpool.h"

class Unit;
class ImageCache;

class GraphicsEngine
{
public:
    GraphicsEngine();
    ~GraphicsEngine();
    void setupCurrentGame();
    void renderScene();
    Uint16 getWidth()
    {
        return width;
    }
    Uint16 getHeight()
    {
        return height;
    }
    SDL_Rect *getMapArea()
    {
        return &maparea;
    }
    void drawVQAFrame(SDL_Surface *frame);
    void clearBuffer();
    void clearScreen();
    /*void postMessage(char *msg) {
        messages->postMessage(msg);
    }*/

    void renderLoading(const std::string& buff, SDL_Surface* logo);

	SDL_Surface *get_SDL_ScreenSurface (void){
		return screen;
	}
    class VideoError {};
private:
	bool MapPosToScreenXY (Uint32 MapPos, Sint16 *ScreenX, Sint16 *ScreenY);
	void DrawSelectionBox (void);
	void DrawMouse (void);
    void DrawMinimap(void);
	void DrawRepairing(void);
	void DrawHealthBars(SDL_Rect dest, SDL_Rect	udest, Uint32 curdpos);
	void DrawL2Overlays (void);
	void DrawFogOfWar(SDL_Rect dest, SDL_Rect src, SDL_Rect udest);
	void DrawMap(SDL_Rect dest, SDL_Rect src, SDL_Rect udest);
    void clipToMaparea(SDL_Rect *dest);
    void clipToMaparea(SDL_Rect *src, SDL_Rect *dest);
    void drawSidebar();
    void drawLine(Sint16 startx, Sint16 starty,
                  Sint16 stopx, Sint16 stopy, Uint16 width, Uint32 colour);
    SDL_Surface* screen;
    SDL_Surface* icon;
    Uint16 width;
    Uint16 height;
    Uint16 tilewidth, tileheight;
    SDL_Rect maparea;
    bool drawFogOfWar;

    Uint32 repair_icon, repairing_icon, sell_icon, map_icon;


	SDL_Rect	oldmouse;

	Uint32	whitepix,
			greenpix,
			yellowpix,
			redpix,
			blackpix;

	std::vector<Uint16> l2overlays;


	Uint32 firstframe;
	Uint32 frames;
	bool clearBack;

    // Minimap zoom factor
    struct {
        Uint8 normal;
        Uint8 max;
    } minizoom;
    // Which zoom is currently used?
    Uint8* mz;
    // Used to avoid SDL_MapRGB in the radar render step.
    std::vector<Uint32> playercolours;
};

#endif
