#include <cstring>
#include <stdexcept>
#include "SDL.h"
#include "fcnc_endian.h"
#include "font.h"
#include "logger.h"
#include "vfs.h"
#include "sdllayer.h"


#define USE_FONTCACHE


#if _MSC_VER && _MSC_VER < 1300
using namespace std;
#else
using std::runtime_error; using std::string; using std::vector;
#endif

#ifdef USE_FONTCACHE
//vector <TFontImage> fontcache;
FontCache		fontCache;
#endif

// Constructor
FontCache::FontCache (void)
{
	// We don't need to do this
	font_cache.empty();
}

// Destructor
FontCache::~FontCache (void)
{
	for (unsigned int i = 0; i < font_cache.size(); i++){
		if (font_cache[i].fontimg != NULL)
			SDL_FreeSurface (font_cache[i].fontimg);
		font_cache[i].fontimg = NULL;
	}
}

// Add a font to the cache
void FontCache::Add (TFontImage Image)
{
	font_cache.push_back (Image);
}

// Add a font to the cache
void FontCache::Add (std::string fontname, SDL_Surface *img,  std::vector<SDL_Rect> chrdest)
{
TFontImage FntImg;

	FntImg.fontname = fontname;
	FntImg.chrdest = chrdest;
	FntImg.fontimg = img;
	font_cache.push_back (FntImg);
}

// Get a font from the cache
bool FontCache::Get (std::string fontname, SDL_Surface **fontimg,  std::vector<SDL_Rect> &chrdest)
{
	*fontimg = NULL;
	chrdest.empty();
	for (unsigned int i = 0; i < font_cache.size(); i++){
		if (font_cache[i].fontname == fontname){
			chrdest = font_cache[i].chrdest;
			*fontimg = font_cache[i].fontimg;
			//printf ("Found font in new cache\n");
			return true;;
		}
	}
	return false;
}


//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************
// START OF FONT CLASS
//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************


Font::Font(const string& fontname) : SHPBase(fontname), fontimg(0)
{
	// Set the image pointer to NULL
	fontimg = NULL;

	this->Load(fontname);
}



Font::~Font() {
#ifndef USE_FONTCACHE
	if (fontimg != NULL)
		SDL_FreeSurface(fontimg);
#endif
}


// Make a gues for the font palette
SDL_Color font_pal[] = {	
		{0, 0, 0, 0x0}, 
		{17, 17, 17, 0x0},
		{34, 34, 34, 0x0},
		{51, 51, 51, 0x0},
		{68, 68, 68, 0x0},
		{85, 85, 85, 0x0},
		{102, 102, 102, 0x0},
		{119, 119, 119, 0x0},
		{136, 136, 136, 0x0},
		{153, 153, 153, 0x0},
		{170, 170, 170, 0x0},
		{187, 187, 187, 0x0},
		{204, 204, 204, 0x0},
		{221, 221, 221, 0x0},
		{238, 238, 238, 0x0},
		{255, 255, 255, 0x0}
};


void Font::reload (void)
{
	this->Load (this->fontname);
}

void Font::Load(std::string FontName) 
{
VFile			*fontfile;
Uint16			wpos,
			hpos,
			cdata,
			nchars;
Uint8			fnheight,
			fnmaxw;
Uint32			fntotalw;
Uint32			ypos,
			i,
			pos,
			curchar;
Uint8			data;
SDL_Surface 		*OrgFonImg = NULL;



	this->fontname = FontName;

#ifdef USE_FONTCACHE
	if (fontCache.Get (this->fontname, &this->fontimg, this->chrdest)){
		return;
	}
#endif //USE_FONTCACHE	

	printf ("%s line %i: Load the font\n", __FILE__, __LINE__);
	fontfile = VFS_Open(fontname.c_str());
	if (0 == fontfile) {
		string s("Unable to load font ");
		s += fontname;
		throw(runtime_error(s));
	}

	fontfile->seekSet(8);
	fontfile->readWord(&wpos, 1);
	fontfile->readWord(&cdata, 1);
	fontfile->readWord(&hpos, 1);
	fontfile->seekCur(2);
	fontfile->readWord(&nchars, 1);
	nchars = SDL_Swap16(nchars); // Yes, even on LE systems.
	fontfile->readByte(&fnheight, 1);
	fontfile->readByte(&fnmaxw, 1);

	nchars++;

	vector<Uint8> wchar(nchars);
	vector<Uint8> hchar(nchars<<1);

	vector<Uint16> dataoffsets(nchars);
	fontfile->readWord(&dataoffsets[0], nchars);

	fontfile->seekSet(wpos);
	fontfile->readByte(&wchar[0], nchars);
	fontfile->seekSet(hpos);
	fontfile->readByte(&hchar[0], nchars<<1);

	chrdest.resize(nchars);

	fntotalw = 0;
	for( i = 0 ; i<nchars; i++ ) {
		chrdest[i].x = fntotalw;
		chrdest[i].y = 0;
		chrdest[i].h = fnheight;
		chrdest[i].w = wchar[i];
		fntotalw += wchar[i];
	}
	vector<Uint8> chardata(fnheight*fntotalw);

	for( curchar = 0; curchar < nchars; curchar++ ) {
		fontfile->seekSet(dataoffsets[curchar]);
		for( ypos = hchar[curchar<<1]; ypos < (Uint32)(hchar[curchar<<1]+hchar[(curchar<<1)+1]); ypos++ ) {
			pos = chrdest[curchar].x+ypos*fntotalw;
			for( i = 0; i < wchar[curchar]; i+=2 ) {
				fontfile->readByte( &data, 1 );
#if 1
				/* Each 4 bits contain a index to the pallete */
				chardata[pos+i] = data&0xf;
				if( i+1<wchar[curchar] )
					chardata[pos+i+1] = (data>>4);
#else
				/* Each 4 bits contain a index to the pallete, convert them to 0 or 1 */
				chardata[pos+i] = (data&0xb)!=0?1:0;
				if( i+1<wchar[curchar] )
					chardata[pos+i+1] = (data>>4)!=0?1:0;
			//printf ("Data = %i\n", chardata[pos+i+1]);
#endif
			}
		}
	}

	SDL_FreeSurface(fontimg);
	fontimg = NULL;

	OrgFonImg = SDL_CreateRGBSurfaceFrom(&chardata[0], fntotalw, fnheight, 8, fntotalw, 0, 0, 0, 0);

	SDL_SetColors(OrgFonImg, font_pal, 0, 16);

	SDL_SetColorKey(OrgFonImg, SDL_SRCCOLORKEY, 0);

	fontimg = SDL_DisplayFormat(OrgFonImg);

	SDL_FreeSurface(OrgFonImg);
	OrgFonImg = NULL;


#ifdef USE_FONTCACHE
	fontCache.Add (this->fontname, this->fontimg,  chrdest);
#endif
	VFS_Close(fontfile);
}


Uint32 Font::getHeight() const {
    return chrdest[0].h;
}

Uint32 Font::calcTextWidth(const std::string& text) const {
    Uint32 wdt = 0;
    Uint32 i;

    for (i = 0; text[i] != '\0'; i++)
        wdt += chrdest[text[i]].w+1;
    return wdt;
}


//
//
// At the moment there is a lot of overhead in reloading the font each time, 
// These new functions are a start at preventing that...
//
//
//
void Font::GetFontColor (SDL_Color FColor, SDL_Color BColor, SDL_Color OrgFntColor, SDL_Color &FntColor)
{
Uint32 perc;

	//calc the percentage:
	perc = ((255 - OrgFntColor.r) * 100) / 255;

	FntColor.r	= (Uint8)((double)FColor.r 	+ ((((double)BColor.r - (double)FColor.r)	* (double)perc)/100));
	FntColor.g	= (Uint8)((double)FColor.g	+ ((((double)BColor.g - (double)FColor.g)	* (double)perc)/100));
	FntColor.b	= (Uint8)((double)FColor.b	+ ((((double)BColor.b - (double)FColor.b)	* (double)perc)/100));

}

/** @Draw a colored text to a surface, uses the colors set with Font::SetColors ()
 * @param text is the text to draw to the surface.
 * @param dest is the destination surface
 * @param startx is the xpos on the surface where the text will be drawn
 * @param starty is the ypos on the surface where the text will be drawn
 * @returns void 
**/

void Font::drawText(const std::string& text, SDL_Surface *dest, Uint32 startx, Uint32 starty)
{
#if 1
Uint32 i;
SDL_Rect destr;
destr.x = startx;
destr.y = starty;
destr.h = chrdest[0].h;
SDL_Color OrgColor;

	SDL_Color	FontColor 	= {255, 255, 255, 0},
			BGColor		= {200, 200, 200, 0};


	for( i = 0; text[i] != '\0'; i++ ) {
		//printf ("Character = %c\n", text[i]);
		SDL_Rect* src_rect = const_cast<SDL_Rect*>(&chrdest[text[i]]);
		destr.w = src_rect->w;
#if 0
		SDL_BlitSurface(fontimg, src_rect, dest, &destr);
#else
		for (int x = 0; x < destr.w; x++){
			for (Uint32 y = 0; y < destr.h; y++){
				// Get the pixel from our source surface
				get_pixel ( fontimg, OrgColor, src_rect->x + x, src_rect->y + y );
				if (OrgColor.r != 0 || OrgColor.g != 0 || OrgColor.b != 0){
					// convert the org color to the correct font color
					GetFontColor (FontColor, BGColor, OrgColor, FontColor);
					// Draw the pixel to our destination surface
					set_pixel ( dest, FontColor, destr.x + x, destr.y + y );
					//set_pixel ( dest, OrgColor, destr.x + x, destr.y + y );
				}
		}
	}
#endif
        destr.x += destr.w+1;
    }
#else
	drawText(text, dest, 0xffffff, startx, starty) 
#endif
}

/** @Draw a colored text to a surface
 * This function gets the background color from the dest surface so
 * !!!this function will not work if you draw over the same text each time without clearing the background !!
 *
 * I think this functions works correctly with the following fonts (don't really know about the rest):
 * type.fnt
 * scorefnt.fnt
 * grad6fnt.fnt
 *
 * @param text is the text to draw to the surface.
 * @param dest is the destination surface
 * @param FGcolor is the font color
 * @param BGcolor is the background of the text
 * @param startx is the xpos on the surface where the text will be drawn
 * @param starty is the ypos on the surface where the text will be drawn
 * @returns void 
**/
void Font::drawText(const std::string& text, SDL_Surface *dest, SDL_Color FGcolor, Uint32 startx, Uint32 starty) 
{
	Uint32 i;
	SDL_Rect destr;
	destr.x = startx;
	destr.y = starty;
	destr.h = chrdest[0].h;
	SDL_Color OrgColor, FontColor, BGcolor;

	for( i = 0; text[i] != '\0'; i++ ) {
		//printf ("Character = %c\n", text[i]);
		SDL_Rect* src_rect = const_cast<SDL_Rect*>(&chrdest[text[i]]);
		destr.w = src_rect->w;
		//SDL_BlitSurface(fontimg, src_rect, dest, &destr);
		for (int x = 0; x < destr.w; x++){
			for (Uint32 y = 0; y < destr.h; y++){
				// Get the pixel from our source surface
				get_pixel ( fontimg, OrgColor, src_rect->x + x, src_rect->y + y );
				if (OrgColor.r != 0 || OrgColor.g != 0 || OrgColor.b != 0){
					get_pixel ( dest, BGcolor, destr.x + x, destr.y + y );
					// convert the org color to the correct font color
					GetFontColor (FGcolor, BGcolor, OrgColor, FontColor);
					// Draw the pixel to our destination surface
					set_pixel ( dest, FontColor, destr.x + x, destr.y + y );
					//set_pixel ( dest, OrgColor, destr.x + x, destr.y + y );
				}
			}
		}
		destr.x += destr.w+1;
	}
}

/** @Draw a colored text to a surface
 * This function gets the background color from the dest surface so
 * !!!***this function will not work if you draw over the same text each time without clearing the background****!!!
 *
 * I think this functions works correctly with the following fonts (don't really know about the rest):
 * type.fnt
 * scorefnt.fnt
 * grad6fnt.fnt
 *
 * @param text is the text to draw to the surface.
 * @param dest is the destination surface
 * @param FGcolor is the font color
 * @param BGcolor is the background of the text
 * @param startx is the xpos on the surface where the text will be drawn
 * @param starty is the ypos on the surface where the text will be drawn
 * @returns void 
**/
void Font::drawText(const std::string& text, SDL_Surface *dest, Uint32 Fcolor, Uint32 startx, Uint32 starty) 
{
SDL_Color FGcolor;

	SDL_GetRGB( Fcolor, dest->format, &FGcolor.r, &FGcolor.g, &FGcolor.b );
	this->drawText(text, dest, FGcolor, startx, starty) ;
}

