#include "SDL.h"
#include "common.h"
#include "config.h"
#include "cpsimage.h"
#include "graphicsengine.h"
#include "logger.h"
#include "menu.h"
#include "cursor.h"
#include "shpimage.h"
#include "vfs.h"
#include "vqa.h"
#include "wsa.h"


#define BUTTON_WITH 240

MenuClass::MenuClass():StartNewGameButton()
{
SDL_Color	palette[256];

	logo		= NULL;
	SDLlogo		= NULL;
	cursorimg	= NULL;
	isDone		= false;
	MenuState	= MENUSTATE_MAIN_MENU;

	display = pc::gfxeng->get_SDL_ScreenSurface ();

	this->loadPal("SNOW", palette);
	SHPBase::setPalette(palette);
	SHPBase::calculatePalettes();

	try {
		logo = new CPSImage("title.cps",1);
	} catch (ImageNotFound&) {
		logger->error("Couldn't load startup graphic\n");
		logo = 0;
	}

	// Get the logo and draw it to the screen
	if (logo != NULL)
		SDLlogo = logo->getImage();
	else
		SDLlogo = NULL;
	this->DrawMenuBackground ();
	SDL_Flip(display);

	// Load the cursor
	cursorimg = new Dune2Image("mouse.shp", -1);
	if (cursorimg == NULL)
		logger->error("Couldn't load cursor image\n");

	my_cursor = cursorimg->getImage(0);

	if (my_cursor == NULL)
		logger->error("Couldn't get cursor sld surface\n");

	if (cursorimg != NULL)
		delete cursorimg;
	cursorimg = NULL;
	SDL_SetColors(my_cursor, SHPBase::getPalette(0), 0, 256);
	SDL_SetColorKey(my_cursor, SDL_SRCCOLORKEY, 0);

	int ButtonXpos	= display->w/2 - BUTTON_WITH/2;
	int ButtonYpos	= 200;

	printf ("Start create button surfaces\n");

	StartNewGameButton.CreateSurface("Start New Game", ButtonXpos, ButtonYpos, BUTTON_WITH, 25 );
	ButtonYpos += 35;

	InternetGameButton.CreateSurface("Internet Game", ButtonXpos, ButtonYpos, BUTTON_WITH, 25 );
	ButtonYpos += 35;

	LoadMissionButton.CreateSurface("Load Mission", ButtonXpos, ButtonYpos, BUTTON_WITH, 25 );
	ButtonYpos += 35;

	MultiplayerGameButton.CreateSurface("Multiplayer Game", ButtonXpos, ButtonYpos, BUTTON_WITH, 25 );
	ButtonYpos += 35;

	IntroAndSneakPeekButton.CreateSurface("Intro & Sneak Peek", ButtonXpos, ButtonYpos, BUTTON_WITH, 25 );
	ButtonYpos += 35;

	ExitGameButton.CreateSurface("Exit Game", ButtonXpos, ButtonYpos, BUTTON_WITH, 25 );

	printf ("End create button surfaces\n");


	//
	// Setup the first popup window
	//
	MissionMenu1.SetupWindow(190, 180, 280, 130);

	MultiPlayerMenu.SetupWindow(0, 0, 640, 400);


	//
	// Setup the buttons for selecting the local player color
	//
	ButtonYpos = 50;
	ButtonXpos = 400;
	ButtonColGreece.setcolor (SHPBase::getColour(display->format, 1, 85));
	ButtonColGreece.CreateSurface(" ", ButtonXpos, ButtonYpos, 25, 25 );
	ButtonColGreece.SetDrawingWindow (&MultiPlayerMenu);
	ButtonXpos += 25;
	ButtonColUssr.setcolor (SHPBase::getColour(display->format, 2, 85));
	ButtonColUssr.CreateSurface(" ", ButtonXpos, ButtonYpos, 25, 25 );
	ButtonColUssr.SetDrawingWindow (&MultiPlayerMenu);
	ButtonXpos += 25;
	ButtonColUk.setcolor (SHPBase::getColour(display->format, 3, 85));
	ButtonColUk.CreateSurface(" ", ButtonXpos, ButtonYpos, 25, 25 );
	ButtonColUk.SetDrawingWindow (&MultiPlayerMenu);
	ButtonXpos += 25;
	ButtonColSpain.setcolor (SHPBase::getColour(display->format, 4, 85));
	ButtonColSpain.CreateSurface(" ", ButtonXpos, ButtonYpos, 25, 25 );
	ButtonColSpain.SetDrawingWindow (&MultiPlayerMenu);
	ButtonXpos += 25;
	ButtonColItaly.setcolor (SHPBase::getColour(display->format, 5, 85));
	ButtonColItaly.CreateSurface(" ", ButtonXpos, ButtonYpos, 25, 25 );
	ButtonColItaly.SetDrawingWindow (&MultiPlayerMenu);
	ButtonXpos += 25;
	ButtonColGermany.setcolor (SHPBase::getColour(display->format, 6, 85));
	ButtonColGermany.CreateSurface(" ", ButtonXpos, ButtonYpos, 25, 25 );
	ButtonColGermany.SetDrawingWindow (&MultiPlayerMenu);
	ButtonXpos += 25;
	ButtonColFranse.setcolor (SHPBase::getColour(display->format, 7, 85));
	ButtonColFranse.CreateSurface(" ", ButtonXpos, ButtonYpos, 25, 25 );
	ButtonColFranse.SetDrawingWindow (&MultiPlayerMenu);
	ButtonXpos += 25;
	ButtonColTurkey.setcolor (SHPBase::getColour(display->format, 8, 85));
	ButtonColTurkey.CreateSurface(" ", ButtonXpos, ButtonYpos, 25, 25 );
	ButtonColTurkey.SetDrawingWindow (&MultiPlayerMenu);

	ButtonColUssr.setButtonState (BUTTON_STATE_DOWN);

	//
	// Draw the buttons on the new window..
	//
	ButtonXpos = 30;
	ButtonYpos = 70;
	AlliesMissionButton.SetDrawingWindow (&MissionMenu1);
	AlliesMissionButton.CreateSurface("Allies", ButtonXpos, ButtonYpos, 90, 25 );
	ButtonXpos = 155;
	RussianMissionButton.SetDrawingWindow (&MissionMenu1);
	RussianMissionButton.CreateSurface("Soviet", ButtonXpos, ButtonYpos, 90, 25 );

	Oke.SetDrawingWindow (&MultiPlayerMenu);
//	Oke.CreateSurface("Oke", 50, 320, 90, 25 );
	Cancel.SetDrawingWindow (&MultiPlayerMenu);
//	Cancel.CreateSurface("Cancel", 500, 320, 90, 25 );

//	MissionMenu1.DrawWindow();

}

MenuClass::~MenuClass()
{
//    int stat;
	if (logo != NULL)
		delete logo;

	if (my_cursor != NULL)
		SDL_FreeSurface(my_cursor);
}

void MenuClass::loadPal(const std::string& paln, SDL_Color *palette)
{
	VFile *palfile;
	int i;

//    string palname = missionData.theater;
	std::string palname = paln;

	printf ("Loading pallete: %s\n", palname.c_str());

	if (palname.length() > 8) {
		palname.insert(8, ".PAL");
	} else {
		palname += ".PAL";
	}

	/* Seek the palette file in the mix */
	palfile = VFS_Open(palname.c_str());

	if (palfile == NULL) {
		logger->error("%s line %i: Unable to open palette file (\"%s\").\n", __FILE__, __LINE__, palname.c_str());
		return;
	}

	/* Load the palette */
	for (i = 0; i < 256; i++) {
		palfile->readByte(&palette[i].r, 1);
		palfile->readByte(&palette[i].g, 1);
		palfile->readByte(&palette[i].b, 1);
		palette[i].r <<= 2;
		palette[i].g <<= 2;
		palette[i].b <<= 2;
	}
	VFS_Close(palfile);
}


void MenuClass::DrawMenuBackground (void)
{
SDL_Rect dest;

	dest.x = 0;
	dest.y = 0;
	dest.w = display->w;
	dest.h = display->h;

	//Black the screen
	static Uint32 blackpix = SDL_MapRGB(display->format, 0, 0, 0);
	SDL_FillRect(display, &dest, blackpix);

	// Draw the menu background pixture
	if (SDLlogo != 0) {
		dest.x = (display->w - SDLlogo->w)/2;
		dest.y = (display->h - SDLlogo->h)/2;
		SDL_BlitSurface(SDLlogo, NULL, display, &dest);
	}
}


void MenuClass::DrawMainMenuButtons (void)
{
	StartNewGameButton.drawbutton ();
	InternetGameButton.drawbutton ();
	LoadMissionButton.drawbutton ();
	MultiplayerGameButton.drawbutton ();
	IntroAndSneakPeekButton.drawbutton ();
	ExitGameButton.drawbutton ();

}

void MenuClass::DrawMousePointer (void)
{
SDL_Rect	dest;
int 		mx, my;

	SDL_GetMouseState(&mx, &my);

//	printf ("Mouse position = %i, %i\n", mx, my);

	//
	// Actually draw the mouse
	//
	dest.x = mx;
	dest.y = my;
	if (dest.x < 0)
		dest.x = 0;
	if (dest.y < 0)
		dest.y = 0;
	dest.w = my_cursor->w;
	dest.h = my_cursor->h;

	SDL_BlitSurface(my_cursor, NULL, display, &dest);
}

void MenuClass::ResetSideColorButtonStates (void)
{
	ButtonColGreece.setButtonState (BUTTON_STATE_UP);
	ButtonColUssr.setButtonState (BUTTON_STATE_UP);
	ButtonColUk.setButtonState (BUTTON_STATE_UP);
	ButtonColSpain.setButtonState (BUTTON_STATE_UP);
	ButtonColItaly.setButtonState (BUTTON_STATE_UP);
	ButtonColGermany.setButtonState (BUTTON_STATE_UP);
	ButtonColFranse.setButtonState (BUTTON_STATE_UP);
	ButtonColTurkey.setButtonState (BUTTON_STATE_UP);
}

void MenuClass::HandleInput (void)
{
SDL_Event	event;
std::string	SelectedMap;

//	if (MenuState == MENUSTATE_MULTIPLAYER_1)


        // Limit fps to ~10
        while ( SDL_PollEvent(&event) && !isDone ) {
		// Handle the buttons for the diffirent menu's
		switch (MenuState){
			case MENUSTATE_MAIN_MENU:
				if (StartNewGameButton.handleMouseEvent (event))
					DrawMousePointer ();
				if (InternetGameButton.handleMouseEvent (event))
					DrawMousePointer ();
				if (LoadMissionButton.handleMouseEvent (event))
					DrawMousePointer ();
				if (MultiplayerGameButton.handleMouseEvent (event))
					DrawMousePointer ();
				if (IntroAndSneakPeekButton.handleMouseEvent (event))
					DrawMousePointer ();
				if (ExitGameButton.handleMouseEvent (event)){
					DrawMousePointer ();
				}
				break;
			case MENUSTATE_MISSION_1:
				if (RussianMissionButton.handleMouseEvent (event))
					DrawMousePointer ();
				if (AlliesMissionButton.handleMouseEvent (event))
					DrawMousePointer ();
				break;
			case MENUSTATE_MULTIPLAYER_1:
				PlayerName.HandleInput(event);
				SideSelection.HandleInput(event);
				ListBox.HandleInput(event);
				if (PlayerName.need_redraw() || SideSelection.need_redraw())
					redraw = true;
				if (Oke.handleMouseEvent (event))
					DrawMousePointer ();
				if (Cancel.handleMouseEvent (event))
					DrawMousePointer ();
				break;
		}

		// Handle the diffirent mouse and keyboard effents
		switch (event.type){

			case SDL_MOUSEBUTTONDOWN:
				redraw = true;
				switch (MenuState){
					case MENUSTATE_MULTIPLAYER_1:
						if (ButtonColGreece.mouseover()){
							this->ResetSideColorButtonStates ();
							ButtonColGreece.setButtonState (BUTTON_STATE_DOWN);
						}else if (ButtonColUssr.mouseover()){
							this->ResetSideColorButtonStates ();
							ButtonColUssr.setButtonState (BUTTON_STATE_DOWN);
						}else if (ButtonColUk.mouseover()){
							this->ResetSideColorButtonStates ();
							ButtonColUk.setButtonState (BUTTON_STATE_DOWN);
						}else if (ButtonColSpain.mouseover()){
							this->ResetSideColorButtonStates ();
							ButtonColSpain.setButtonState (BUTTON_STATE_DOWN);
						}else if (ButtonColItaly.mouseover()){
							this->ResetSideColorButtonStates ();
							ButtonColItaly.setButtonState (BUTTON_STATE_DOWN);
						}else if (ButtonColGermany.mouseover()){
							this->ResetSideColorButtonStates ();
							ButtonColGermany.setButtonState (BUTTON_STATE_DOWN);
						}else if (ButtonColFranse.mouseover()){
							this->ResetSideColorButtonStates ();
							ButtonColFranse.setButtonState (BUTTON_STATE_DOWN);
						}else if (ButtonColTurkey.mouseover()){
							this->ResetSideColorButtonStates ();
							ButtonColTurkey.setButtonState (BUTTON_STATE_DOWN);
						}
						break;
				}
				break;

			case SDL_MOUSEBUTTONUP:
				redraw = true;
				if (event.button.button == SDL_BUTTON_LEFT){
					switch (MenuState){
						case MENUSTATE_MAIN_MENU:
							//printf ("Left mouse -->\n");
							if (StartNewGameButton.mouseover()){
								MenuState = MENUSTATE_MISSION_1;
								//isDone=true;
							}else if (ExitGameButton.mouseover()){
								isDone=true;
								pc::quit = true;
							}else if (MultiplayerGameButton.mouseover()){
								MenuState = MENUSTATE_MULTIPLAYER_1;
								Oke.SetDrawingWindow (&MultiPlayerMenu);
								Oke.CreateSurface("Oke", 50, 320, 90, 25 );
								Cancel.SetDrawingWindow (&MultiPlayerMenu);
								Cancel.CreateSurface("Cancel", 500, 320, 90, 25 );


							}else if (IntroAndSneakPeekButton.mouseover()){
								VQAMovie mov("prolog");
								mov.play();
							}
							break;
						case MENUSTATE_MISSION_1:
							if (RussianMissionButton.mouseover()){
								printf ("RussianMission selected\n");
								pc::Config.UseFogOfWar	= true;
								pc::MissionsMapdata->GetNodMissionMap (0, SelectedMap);
								printf ("First nod mission = %s\n", SelectedMap.c_str());
								pc::Config.mapname	= SelectedMap;//"SCG01EA";	// First nod mission
								pc::Config.gamemode	= 0;		// single player (missions)
								pc::Config.mside	= "nod"; 	//"badguy";
								pc::Config.cside	= "multi";
								pc::Config.side_colour	= "red";
								pc::Config.pause = false;
								pc::Config.quit_mission = false;
								isDone=true;
							}else if (AlliesMissionButton.mouseover()){
								printf ("AlliesMission selected\n");
								pc::Config.UseFogOfWar	= true;
								pc::MissionsMapdata->GetGdiMissionMap (0, SelectedMap);
								printf ("First gdi mission = %s\n", SelectedMap.c_str());
								isDone=true;
								pc::Config.mapname	= SelectedMap;	//"SCU01EA";	// First allied mission??
								pc::Config.gamemode	= 0;		// single player (missions)
								pc::Config.mside	= "gdi"; //"goodguy";
								pc::Config.cside	= "multi";
								pc::Config.side_colour	= "yellow";
								pc::Config.pause = false;
								pc::Config.quit_mission = false;

							}
							break;
						case MENUSTATE_MULTIPLAYER_1:
							//if (ListBox.MouseOverArrowDown ()){
							//	ListBox.ScrollDown();
							//	redraw = true;
							//}
							if (Cancel.mouseover()){
								MenuState = MENUSTATE_MAIN_MENU;
								redraw = true;
							}


							// Supported colors
							//"yellow"
							//"red"
							//"blue"
							//"orange"
							//"green"
							//"turquoise"
							if (Oke.mouseover()){
								printf ("Selected map nr = %i\n", ListBox.GetSelectionIndex ());
								MultiPlayerMaps.GetMapName (ListBox.GetSelectionIndex (), pc::Config.mapname); 
								// Printf ("For some reason the game crashes when skirmisch is selected??
								pc::Config.UseFogOfWar	= true;
                				pc::Config.playernum	= 1;	// Local player player number
								pc::Config.gamemode 	= 1;	// Skermisch
								pc::Config.totalplayers = 2;
								if (SideSelection.selected() == 0 || SideSelection.selected() == 2)
									pc::Config.mside	= "nod";
								else
									pc::Config.mside	= "gdi";
								pc::Config.cside	= "goodguy";
								pc::Config.side_colour	= "red";
								//pc::Config.side_colour	= "blue"; //"orange"; //"red";
								pc::Config.startMoney	= 10000;
								pc::Config.nick		= PlayerName.getText();
								pc::Config.pause = false;
								pc::Config.quit_mission = false;
								isDone=true;
								printf ("Mapname = %s\n", pc::Config.mapname.c_str());

								if (ButtonColGreece.getButtonState () == BUTTON_STATE_DOWN){
									pc::Config.side_colour	= "blue";
								}else if (ButtonColUssr.getButtonState () == BUTTON_STATE_DOWN){
									pc::Config.side_colour	= "red";
								}else if (ButtonColUk.getButtonState () == BUTTON_STATE_DOWN){
									pc::Config.side_colour	= "green";
								}else if (ButtonColSpain.getButtonState () == BUTTON_STATE_DOWN){
									pc::Config.side_colour	= "yellow";
								}else if (ButtonColItaly.getButtonState () == BUTTON_STATE_DOWN){
									pc::Config.side_colour	= "orange";
								}else if (ButtonColGermany.getButtonState () == BUTTON_STATE_DOWN){
									pc::Config.side_colour	= "gray";
								}else if (ButtonColFranse.getButtonState () == BUTTON_STATE_DOWN){
									pc::Config.side_colour	= "turquoise";
								}else if (ButtonColTurkey.getButtonState () == BUTTON_STATE_DOWN){
									pc::Config.side_colour	= "darkred";
								}

							}
							break;
					}

					//if (ListBox.MouseOverArrowUp ()){
					//	ListBox.ScrollUp();
					//	redraw = true;
					//}
				}
				break;
        		case SDL_KEYDOWN:
				switch (event.key.keysym.sym){
					case SDLK_ESCAPE:
						printf ("Escape key\n");
						if (MenuState == MENUSTATE_MISSION_1 || MenuState == MENUSTATE_MULTIPLAYER_1){
							MenuState = MENUSTATE_MAIN_MENU;
							redraw = true;
						}else{
							isDone=true;
							pc::quit = true;
						}
						break;
					default:
						break;
				}
				break;
		}
	}
}

int MenuClass::HandleMenu(void)
{

int	mx,
	my,
	old_mx = 0,
	old_my = 100;


std::string		TmpString;
int			i = 0;

TCheckBox		CheckBox;

Font			MenuFont("type.fnt");

SDL_Color		Fcolor;


//TTextBox	PlayerName;

	// Setup the font color
	Fcolor.r = 255;
	Fcolor.g = 255;
	Fcolor.b = 255;

//	printf ("Start setup multiplayer listbox\n");
	
	// Setup the multi player listbox with all the multiplayer maps
	while (MultiPlayerMaps.GetMapDescription (i, TmpString)){
		ListBox.AddString (TmpString);
		i++;
	}

//	printf ("End setup mulitplayer listbox\n");

	redraw = true;

	MenuFont.drawText("Your color", MultiPlayerMenu.GetWindowSurface (), Fcolor, 400, 30) ; //drawText("Your color", ListBox.geDrawingSurface(), 100, 20);
	MenuFont.drawText("Your name", MultiPlayerMenu.GetWindowSurface (), Fcolor, 50, 30) ; //drawText("Your color", ListBox.geDrawingSurface(), 100, 20);
	MenuFont.drawText("Your side", MultiPlayerMenu.GetWindowSurface (), Fcolor, 220, 30) ; //drawText("Your color", ListBox.geDrawingSurface(), 100, 20);


	// should be: Russia, England, Ukraine, Germany, France
	SideSelection.AddEntry("Russia");
	SideSelection.AddEntry("England");
	SideSelection.AddEntry("Ukraine");
	SideSelection.AddEntry("Germany");
	SideSelection.AddEntry("France");
//	SideSelection.AddEntry("Allies");

	while( !isDone ) {

		SDL_GetMouseState(&mx, &my);

		this->HandleInput ();

//		if (MenuState == MENUSTATE_MULTIPLAYER_1 && PlayerName.need_redraw()){
//			redraw = true;
//		}

		if (old_mx != mx || old_my != my || redraw){

			//printf ("Redraw\n");

			// Draw the background pixture
			this->DrawMenuBackground ();

			this->DrawMainMenuButtons ();

			if (MenuState == MENUSTATE_MISSION_1){
				// first draw the buttons on the window, 
				// than draw the window to the screen
				RussianMissionButton.drawbutton ();
				AlliesMissionButton.drawbutton ();
				MissionMenu1.DrawWindow();
			}else if (MenuState == MENUSTATE_MULTIPLAYER_1){
				// first draw the buttons on the window, 
				// than draw the window to the screen
				Oke.drawbutton ();
				Cancel.drawbutton ();
				ButtonColGreece.drawbutton ();
				ButtonColUssr.drawbutton ();
				ButtonColUk.drawbutton ();
				ButtonColSpain.drawbutton ();
				ButtonColItaly.drawbutton ();
				ButtonColGermany.drawbutton ();
				ButtonColFranse.drawbutton ();
				ButtonColTurkey.drawbutton ();
				MultiPlayerMenu.DrawWindow();
				PlayerName.Draw(50,50);

				ListBox.DrawListBox (130, 100);

				SideSelection.Draw (220,50);
				//CheckBox.Draw(30, 30);
			}

			this->DrawMousePointer ();

			SDL_Flip(display);

			old_mx = mx;
			old_my = my;
			redraw = false;



		}
    }

    return 0;
}
