#include "SDL.h"
#include "common.h"
#include "config.h"
#include "cpsimage.h"
#include "graphicsengine.h"
#include "logger.h"
#include "pause_menu.h"
#include "cursor.h"
#include "sidebar.h"
#include "shpimage.h"
#include "vfs.h"
#include "vqa.h"
#include "wsa.h"
#include "soundengine.h"

PauseMenuClass::PauseMenuClass():MenuFont("type.fnt")
{

	// Setup the font color
	Font_color.r = 255;
	Font_color.g = 255;
	Font_color.b = 255;

	display = pc::gfxeng->get_SDL_ScreenSurface ();

	// Load the cursor
	cursorimg = new Dune2Image("mouse.shp", -1);
	if (cursorimg == NULL)
		logger->error("Couldn't load cursor image\n");

	my_cursor = cursorimg->getImage(0);

	//
	// Setup the first popup window
	//
	PauseWindow.SetupWindow(190, 180, 280, 130);

	//
	// Draw the buttons on the new window..
	//
	ExitButton.SetDrawingWindow (&PauseWindow);
	ExitButton.CreateSurface("Exit", 155, 70, 90, 25 );
	ContinueButton.SetDrawingWindow (&PauseWindow);
	ContinueButton.CreateSurface("Continue", 30, 70, 90, 25 );

	MenuFont.drawText("Game paused", PauseWindow.GetWindowSurface (), Font_color, 90, 30) ;

//	MissionMenu1.DrawWindow();

}

PauseMenuClass::~PauseMenuClass()
{

	if (my_cursor != NULL)
		SDL_FreeSurface(my_cursor);
}

void PauseMenuClass::DrawMousePointer (void)
{
SDL_Rect	dest;
int 		mx, my;

	SDL_GetMouseState(&mx, &my);

//	printf ("Mouse position = %i, %i\n", mx, my);

	//
	// Actually draw the mouse
	//
	dest.x = mx;
	dest.y = my;
	if (dest.x < 0)
		dest.x = 0;
	if (dest.y < 0)
		dest.y = 0;
	dest.w = my_cursor->w;
	dest.h = my_cursor->h;

	SDL_BlitSurface(my_cursor, NULL, display, &dest);
}


void PauseMenuClass::DrawMenuBackground (void)
{
SDL_Rect 	dest,
    		*tabpos;
SDL_Surface* sbar;

	pc::sfxeng->PauseMusic();

	dest.x = 0;
	dest.y = 30;
	dest.w = display->w;
	dest.h = display->h -30;

	if ( pc::sidebar->getVisible() ){
    	tabpos = pc::sidebar->getTabLocation();
		dest.w -= tabpos->w;
	}

	//Black the screen
	static Uint32 blackpix = SDL_MapRGB(display->format, 0, 0, 0);
	SDL_FillRect(display, &dest, blackpix);

	// Draw the sidebar (if needed)
	if ( pc::sidebar->getVisible() ){
		tabpos = pc::sidebar->getTabLocation();
        dest.y = tabpos->h;
        dest.x = display->w - tabpos->w;
        dest.w = tabpos->w;
        dest.h = display->h-tabpos->h;

		sbar = pc::sidebar->getSidebarImage(dest);
//		dest.x = display->w - tabpos->w;
//		dest.y = 0;
//		dest.w = sbar->w;
//		dest.h = sbar->h;
		SDL_BlitSurface( sbar, NULL, display, &dest);
	}


}


void PauseMenuClass::HandleInput (void)
{
SDL_Event	event;

	while ( SDL_PollEvent(&event) ) {

		// Pass events on to diffirent menu items
		if (ExitButton.handleMouseEvent (event))
			DrawMousePointer ();
		if (ContinueButton.handleMouseEvent (event))
			DrawMousePointer ();


		// Handle the diffirent mouse and keyboard effents
		switch (event.type){

			case SDL_MOUSEBUTTONDOWN:
				redraw = true;
				break;

			case SDL_MOUSEBUTTONUP:
				redraw = true;
				if (event.button.button == SDL_BUTTON_LEFT){
					if (ExitButton.mouseover()){
						// exit to main menu
						//printf ("%s line %i: exit\n", __FILE__, __LINE__);
						pc::Config.quit_mission = true;
						pc::Config.pause = false;
						pc::sfxeng->ResumeLoopedSound(-1);
						for (int i = 0; i < p::ppool->getNumPlayers(); i++)
							p::ppool->getPlayer(i)->resumeBuilding();
						//p::ppool->getLPlayer()->resumeBuilding();
					}
					if (ContinueButton.mouseover()){
						// Continue the game
						//printf ("%s line %i: continue\n", __FILE__, __LINE__);
						pc::Config.pause = false;
						pc::sfxeng->ResumeLoopedSound(-1);
						for (int i = 0; i < p::ppool->getNumPlayers(); i++)
							p::ppool->getPlayer(i)->resumeBuilding();
						//p::ppool->getLPlayer()->resumeBuilding();
					}
				}
				break;
			case SDL_KEYDOWN:
				if (event.key.keysym.sym == SDLK_ESCAPE){
					pc::Config.pause = false;
				}
				break;
		}
	}
}

int PauseMenuClass::HandleMenu(void)
{
int				mx,
				my,
				old_mx = 0,
				old_my = 100;
std::string		TmpString;
TCheckBox		CheckBox;



	redraw = true;

	pc::sfxeng->PauseLoopedSound(-1);

	for (int i = 0; i < p::ppool->getNumPlayers(); i++)
		p::ppool->getPlayer(i)->pauseBuilding();

	SDL_GetMouseState(&mx, &my);

	this->HandleInput ();

	if (old_mx != mx || old_my != my || redraw){

		DrawMenuBackground ();

		// Draw the buttons on the window
		ExitButton.drawbutton ();
		ContinueButton.drawbutton ();

		// Draw the window (whith the buttons on it) to the screen
		PauseWindow.DrawWindow();

		this->DrawMousePointer ();

		SDL_Flip(display);

		old_mx = mx;
		old_my = my;
		redraw = false;
	}
    return 0;
}
