#include "message.h"
#include "SDL.h"
#include "logger.h"
#include <functional>

using std::list;

namespace
{

struct drawMessage : std::unary_function<void, Message>
{
    drawMessage(Font& font, SDL_Surface* textimg, Uint32& msgy) : font(font), textimg(textimg), msgy(msgy) {}
    void operator()(const Message& msg) {
        font.drawText(msg.getMessage(), textimg, 0xffffff, 0, msgy);
        msgy += font.getHeight()+1;
    }
    Font& font;
    SDL_Surface* textimg;
    Uint32& msgy;
};

}

MessagePool::MessagePool() : updated(false), textimg(0), msgfont(/*"scorefnt.fnt"*/"type.fnt"), width(0)
{
//    const SDL_Color White 	= {255, 255, 255, 0};
//    const SDL_Color Gray	= {100, 100, 100, 0};

//    msgfont.SetColors (White, Gray);

}

MessagePool::~MessagePool()
{
    SDL_FreeSurface(textimg);
}

SDL_Surface *MessagePool::getMessages()
{
    Uint32 curtick = SDL_GetTicks();
    SDL_Rect dest;

    if (!updated) {
        /// @TODO Replace this with STL magic
        for (list<Message>::iterator i = msglist.begin(); i != msglist.end();) {
            if (i->expired(curtick)) {
                updated = true;
                i = msglist.erase(i);
            } else {
                ++i;
            }
        }
    }
    if (!updated) { // Check again now we've possibly changed the value in the above loop
        return textimg;
    }
    updated = false;
    SDL_FreeSurface(textimg);
    textimg = NULL;
    if (msglist.empty()) {
        return textimg;
    }
    textimg = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCCOLORKEY, width,
            static_cast<int>(msglist.size()*(msgfont.getHeight()+1)), 16, 0, 0, 0, 0);
    dest.x = 0;
    dest.y = 0;
    dest.w = textimg->w;
    dest.h = textimg->h;
    SDL_FillRect(textimg, &dest, 0);
    SDL_SetColorKey(textimg, SDL_SRCCOLORKEY, 0);
    Uint32 msgy = 0;
    drawMessage dm(msgfont, textimg, msgy);
    for_each(msglist.begin(), msglist.end(), dm);
    return textimg;
}

void MessagePool::postMessage(const std::string& msg)
{
    msglist.push_back(Message(msg, SDL_GetTicks()+10000));
    updated = true;
}


void MessagePool::clear()
{
    msglist.clear();
    updated = true;
}

void MessagePool::refresh()
{
    //Reload the font
    msgfont.reload();

    updated = true;
    getMessages(); //Forces the SDL_Surface to reload
}
