#include "SDL.h"
#include "logger.h"


//#define _15BIT(r,g,b) (((r&248)<<7) + ((g&248)<<2) + (b>>3))
//#define _16BIT(r,g,b) (((r&248)<<8) + ((g&252)<<3) + (b>>3))

/** @Get a pixel from a SDL surface
 * @param Surface the sdl surface to get the pixel from.
 * @param color the color gotten from the sdl surface
 * @param x x-pos in the picture to get the pixel color form
 * @param y y-pos in the picture to get the pixel color form
 * @returns void 
**/
void get_pixel ( SDL_Surface* Surface, Uint32 &color, Uint32 x, Uint32 y )
{
//Uint32	color = 0;
Uint8	*ubuff8;
Uint16	*ubuff16; 
Uint32	*ubuff32;

	color = 0;

	/* How we draw the pixel depends on the bitdepth */
	switch(Surface->format->BytesPerPixel) 
	{
		case 1:
			ubuff8 = (Uint8*) Surface->pixels;
			ubuff8 += (y * Surface->pitch) + x; 
			//*ubuff8 = (Uint8) color;
			color = *ubuff8;
			break;

		case 2:
			ubuff8 = (Uint8*) Surface->pixels;
			ubuff8 += (y * Surface->pitch) + (x*2);
			ubuff16 = (Uint16*) ubuff8;
			//*ubuff16 = (Uint16) color; 
			color = *ubuff16;
			break;

		case 3:
			ubuff8 = (Uint8*) Surface->pixels;
			ubuff8 += (y * Surface->pitch) + (x*3);
			color = 0;
			#if SDL_BYTEORDER == SDL_LIL_ENDIAN
				color |= ubuff8[2] << 16;
				color |= ubuff8[1] << 8;
				color |= ubuff8[0];
			#else 
				color |= ubuff8[0] << 16;
				color |= ubuff8[1] << 8;
				color |= ubuff8[2];
			#endif
			break;

		case 4:
			ubuff8 = (Uint8*) Surface->pixels;
			ubuff8 += (y*Surface->pitch) + (x*4);
			ubuff32 = (Uint32*)ubuff8;
			color = *ubuff32;
			break;

		default:
			logger->error("Error: Unknown color depth in surface\n");
	}
//	return color;
}

/** @Get a pixel from a SDL surface
 * @param Surface the sdl surface to get the pixel from.
 * @param RGBcolor the color gotten from the sdl surface
 * @param x x-pos in the picture to get the pixel color form
 * @param y y-pos in the picture to get the pixel color form
 * @returns void 
**/
void get_pixel ( SDL_Surface* Surface, SDL_Color &RGBcolor, Uint32 x, Uint32 y )
{
Uint32	color = 0;

	get_pixel ( Surface, color, x, y );
	SDL_GetRGB( color, Surface->format, &RGBcolor.r, &RGBcolor.g, &RGBcolor.b );
}

/** @Set a pixel to a color in a sdl surface
 * @param Surface the sdl surface to get the pixel from.
 * @param color the color gotten from the sdl surface (Uint32 color)
 * @param x x-pos in the picture to get the pixel color form
 * @param y y-pos in the picture to get the pixel color form
 * @returns void 
**/
void set_pixel ( SDL_Surface* Surface, Uint32 color, Uint32 x, Uint32 y )
{
Uint8	*ubuff8;
Uint16	*ubuff16; 
Uint32	*ubuff32;

	/* How we draw the pixel depends on the bitdepth */
	switch(Surface->format->BytesPerPixel) 
	{
		case 1: 
			ubuff8 = (Uint8*) Surface->pixels;
			ubuff8 += (y * Surface->pitch) + x; 
			*ubuff8 = (Uint8) color;
			break;

		case 2:
			ubuff8 = (Uint8*) Surface->pixels;
			ubuff8 += (y * Surface->pitch) + (x*2);
			ubuff16 = (Uint16*) ubuff8;
			*ubuff16 = (Uint16) color; 
			break;

		case 3:
			ubuff8 = (Uint8*) Surface->pixels;
			ubuff8 += (y * Surface->pitch) + (x*3);
			color = 0;
			if(SDL_BYTEORDER == SDL_LIL_ENDIAN) {
				ubuff8[2] = (color & 0xFF0000) >> 16;
				ubuff8[1] = (color & 0x00FF00) >> 8;
				ubuff8[0] = (color & 0x0000FF);
			} else {
				ubuff8[0] = (color & 0xFF0000) >> 16;
				ubuff8[1] = (color & 0x00FF00) >> 8;
				ubuff8[2] = (color & 0x0000FF);	
			}
			break;

		case 4:
			ubuff8 = (Uint8*) Surface->pixels;
			ubuff8 += (y*Surface->pitch) + (x*4);
			ubuff32 = (Uint32*)ubuff8;
			*ubuff32 = color;
			break;

		default:
			logger->error("Error: Unknown color depth in surface\n");
	}
}

/** @Set a pixel to a color in a sdl surface
 * @param Surface the sdl surface to get the pixel from.
 * @param RGBcolor the color gotten from the sdl surface (SDL_Color color)
 * @param x x-pos in the picture to get the pixel color form
 * @param y y-pos in the picture to get the pixel color form
 * @returns void 
**/
void set_pixel ( SDL_Surface* Surface, SDL_Color RGBcolor, Uint32 x, Uint32 y )
{
	Uint32 color = SDL_MapRGB( Surface->format, RGBcolor.r, RGBcolor.g, RGBcolor.b );
	set_pixel ( Surface, color, x, y );
}


/** @Draw a rectangle outline on a sdl surface
 * @param Surface the sdl surface to draw the rectangle on.
 * @param color the color of the rectangle to draw
 * @param x x-pos of the rectangle to draw
 * @param y y-pos of the rectangle to draw
 * @param wdith the width of the rectangle to draw
 * @param height the heigth of the rectangle to draw
 * @param lnpx the line weigth of the square to draw
 * @returns void 
**/
void draw_rectangle ( SDL_Surface* Surface, Uint32 color, Uint16 x, Uint16 y, Uint16 width, Uint16 height, Uint8 lnpx )
{
	if (Surface == NULL)
		return;

	// Draw the top line
	for (int i = 0; i < width; i++){
		for (int j = 0; j < lnpx; j++){
			set_pixel ( Surface, color, x + i, y + j );
		}
	}

	// Draw the bottum line
	for (int i = 0; i < width; i++){
		for (int j = 0; j < lnpx; j++){
			set_pixel ( Surface, color, x + i, y + height - j - 1 );
		}
	}

	// Draw the left line
	for (int i = 0; i < height; i++){
		for (int j = 0; j < lnpx; j++){
			set_pixel ( Surface, color, x + j, y + i );
		}
	}

	// Draw the rigth line
	for (int i = 0; i < height; i++){
		for (int j = 0; j < lnpx; j++){
			set_pixel ( Surface, color, x + width - j - 1, y + i );
		}
	}
}

/** @Draw a rectangle outline on a sdl surface
 * @param Surface the sdl surface to draw the rectangle on.
 * @param RGBcolor the color of the rectangle to draw
 * @param x x-pos of the rectangle to draw
 * @param y y-pos of the rectangle to draw
 * @param wdith the width of the rectangle to draw
 * @param height the heigth of the rectangle to draw
 * @param lnpx the line weigth of the square to draw
 * @returns void 
**/
void draw_rectangle ( SDL_Surface* Surface, SDL_Color RGBcolor, Uint16 x, Uint16 y, Uint16 width, Uint16 height, Uint8 lnpx )
{
	Uint32 color = SDL_MapRGB( Surface->format, RGBcolor.r, RGBcolor.g, RGBcolor.b );
	draw_rectangle ( Surface, color, x, y, width, height, lnpx );
}


/** @Draw a solid rectangle on a sdl surface
 * @param Surface the sdl surface to draw the rectangle on.
 * @param color the color of the rectangle to draw
 * @param x x-pos of the rectangle to draw
 * @param y y-pos of the rectangle to draw
 * @param wdith the width of the rectangle to draw
 * @param height the heigth of the rectangle to draw
 * @returns void 
**/
void draw_solidrectangle ( SDL_Surface* Surface, Uint32 color, Uint16 x, Uint16 y, Uint16 width, Uint16 height )
{
	if (Surface == NULL)
		return;

	for (int i = 0; i < width; i++){
		for (int j = 0; j < height; j++){
			set_pixel ( Surface, color, i, j );
		}
	}
}

/** @Draw a solid rectangle on a sdl surface
 * @param Surface the sdl surface to draw the rectangle on.
 * @param RGBcolor the color of the rectangle to draw
 * @param x x-pos of the rectangle to draw
 * @param y y-pos of the rectangle to draw
 * @param wdith the width of the rectangle to draw
 * @param height the heigth of the rectangle to draw
 * @returns void 
**/
void draw_solidrectangle ( SDL_Surface* Surface, SDL_Color RGBcolor, Uint16 x, Uint16 y, Uint16 width, Uint16 height )
{
	Uint32 color = SDL_MapRGB( Surface->format, RGBcolor.r, RGBcolor.g, RGBcolor.b );
	draw_solidrectangle ( Surface, color, x, y, width, height );
}
